@@ -3,7 +3,7 @@
  * Copyright (c) 1985 Supoj Sutanthavibul
  * Copyright (c) 1991 Micah Beck
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2020 Thomas Loimer
+ * Parts Copyright (c) 2015-2023 Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -196,52 +196,52 @@ arc_bound(F_arc *arc, int *xmin, int *ymin, int *xmax, int *ymax)
 	sy = min(arc->point[2].y, sy);
 
 	if (arc->direction == 1) { /* counter clockwise */
-	    if (alpha > beta) {
-		if (alpha <= 0 || 0 <= beta)
-		    bx = (int)(arc->center.x + radius + 1.0);
-		if (alpha <= M_PI_2 || M_PI_2 <= beta)
-		    sy = (int)(arc->center.y - radius - 1.0);
-		if (alpha <= M_PI || M_PI <= beta)
-		    sx = (int)(arc->center.x - radius - 1.0);
-		if (alpha <= Two_seventy_deg || Two_seventy_deg <= beta)
-		    by = (int)(arc->center.y + radius + 1.0);
-	    } else {
-		if (0 <= beta && alpha <= 0)
-		    bx = (int)(arc->center.x + radius + 1.0);
-		if (M_PI_2 <= beta && alpha <= M_PI_2)
-		    sy = (int)(arc->center.y - radius - 1.0);
-		if (M_PI <= beta && alpha <= M_PI)
-		    sx = (int)(arc->center.x - radius - 1.0);
-		if (Two_seventy_deg <= beta && alpha <= Two_seventy_deg)
-		    by = (int)(arc->center.y + radius + 1.0);
-	    }
+		if (alpha > beta) {
+			if (alpha <= 0 || 0 <= beta)
+				bx = (int)(arc->center.x + radius + 1.0);
+			if (alpha <= M_PI_2 || M_PI_2 <= beta)
+				sy = (int)(arc->center.y - radius - 1.0);
+			if (alpha <= M_PI || M_PI <= beta)
+				sx = (int)(arc->center.x - radius - 1.0);
+			if (alpha <= Two_seventy_deg || Two_seventy_deg <= beta)
+				by = (int)(arc->center.y + radius + 1.0);
+		} else {
+			if (0 <= beta && alpha <= 0)
+				bx = (int)(arc->center.x + radius + 1.0);
+			if (M_PI_2 <= beta && alpha <= M_PI_2)
+				sy = (int)(arc->center.y - radius - 1.0);
+			if (M_PI <= beta && alpha <= M_PI)
+				sx = (int)(arc->center.x - radius - 1.0);
+			if (Two_seventy_deg <= beta && alpha <= Two_seventy_deg)
+				by = (int)(arc->center.y + radius + 1.0);
+		}
 	} else {	/* clockwise	*/
-	    if (alpha > beta) {
-		if (beta <= 0 && 0 <= alpha)
-		    bx = (int)(arc->center.x + radius + 1.0);
-		if (beta <= M_PI_2 && M_PI_2 <= alpha)
-		    sy = (int)(arc->center.y - radius - 1.0);
-		if (beta <= M_PI && M_PI <= alpha)
-		    sx = (int)(arc->center.x - radius - 1.0);
-		if (beta <= Two_seventy_deg && Two_seventy_deg <= alpha)
-		    by = (int)(arc->center.y + radius + 1.0);
-	    } else {
-		if (0 <= alpha || beta <= 0)
-		    bx = (int)(arc->center.x + radius + 1.0);
-		if (M_PI_2 <= alpha || beta <= M_PI_2)
-		    sy = (int)(arc->center.y - radius - 1.0);
-		if (M_PI <= alpha || beta <= M_PI)
-		    sx = (int)(arc->center.x - radius - 1.0);
-		if (Two_seventy_deg <= alpha || beta <= Two_seventy_deg)
-		    by = (int)(arc->center.y + radius + 1.0);
-	    }
+		if (alpha > beta) {
+			if (beta <= 0 && 0 <= alpha)
+				bx = (int)(arc->center.x + radius + 1.0);
+			if (beta <= M_PI_2 && M_PI_2 <= alpha)
+				sy = (int)(arc->center.y - radius - 1.0);
+			if (beta <= M_PI && M_PI <= alpha)
+				sx = (int)(arc->center.x - radius - 1.0);
+			if (beta <= Two_seventy_deg && Two_seventy_deg <= alpha)
+				by = (int)(arc->center.y + radius + 1.0);
+		} else {
+			if (0 <= alpha || beta <= 0)
+				bx = (int)(arc->center.x + radius + 1.0);
+			if (M_PI_2 <= alpha || beta <= M_PI_2)
+				sy = (int)(arc->center.y - radius - 1.0);
+			if (M_PI <= alpha || beta <= M_PI)
+				sx = (int)(arc->center.x - radius - 1.0);
+			if (Two_seventy_deg <= alpha || beta <= Two_seventy_deg)
+				by = (int)(arc->center.y + radius + 1.0);
+		}
 	}
 	/* if pie-wedge type, account for the center point */
 	if(arc->type == T_PIE_WEDGE_ARC) {
-	    sx = min((int)arc->center.x, sx);
-	    bx = max((int)arc->center.x, bx);
-	    sy = min((int)arc->center.y, sy);
-	    by = max((int)arc->center.y, by);
+		sx = min((int)arc->center.x, sx);
+		bx = max((int)arc->center.x, bx);
+		sy = min((int)arc->center.y, sy);
+		by = max((int)arc->center.y, by);
 	}
 
 	*xmin = sx;
@@ -269,96 +269,117 @@ compound_bound(F_compound *compound, int *xmin, int *ymin, int *xmax, int *ymax,
 	llx = lly =  10000000;
 	urx = ury = -10000000;
 	while(compound != NULL) {
-	    for (a = compound->arcs; a != NULL; a = a->next) {
-		if (adjust_boundingbox && !depth_filter(a->depth))
-		  continue;
-		arc_bound(a, &sx, &sy, &bx, &by);
-		half_wd = (a->thickness + 1) / 2;
-		if (first) {
-		    first = 0;
-		    llx = sx - half_wd; lly = sy - half_wd;
-		    urx = bx + half_wd; ury = by + half_wd;
-		} else {
-		    llx = min(llx, sx - half_wd); lly = min(lly, sy - half_wd);
-		    urx = max(urx, bx + half_wd); ury = max(ury, by + half_wd);
+		for (a = compound->arcs; a != NULL; a = a->next) {
+			if (adjust_boundingbox && !depth_filter(a->depth))
+				continue;
+			arc_bound(a, &sx, &sy, &bx, &by);
+			half_wd = (a->thickness + 1) / 2;
+			if (first) {
+				first = 0;
+				llx = sx - half_wd;
+				lly = sy - half_wd;
+				urx = bx + half_wd;
+				ury = by + half_wd;
+			} else {
+				llx = min(llx, sx - half_wd);
+				lly = min(lly, sy - half_wd);
+				urx = max(urx, bx + half_wd);
+				ury = max(ury, by + half_wd);
+			}
 		}
-	    }
 
-	    if (compound->compounds) {
-		compound_bound(compound->compounds, &sx, &sy, &bx, &by, include);
-		if (first) {
-		    first = 0;
-		    llx = sx; lly = sy;
-		    urx = bx; ury = by;
-		} else {
-		    llx = min(llx, sx); lly = min(lly, sy);
-		    urx = max(urx, bx); ury = max(ury, by);
+		if (compound->compounds) {
+			compound_bound(compound->compounds, &sx, &sy, &bx, &by,
+					include);
+			if (first) {
+				first = 0;
+				llx = sx;
+				lly = sy;
+				urx = bx;
+				ury = by;
+			} else {
+				llx = min(llx, sx);
+				lly = min(lly, sy);
+				urx = max(urx, bx);
+				ury = max(ury, by);
+			}
 		}
-	    }
 
-	    for (e = compound->ellipses; e != NULL; e = e->next) {
-		if (adjust_boundingbox && !depth_filter(e->depth))
-		  continue;
-		ellipse_bound(e, &sx, &sy, &bx, &by);
-		if (first) {
-		    first = 0;
-		    llx = sx; lly = sy;
-		    urx = bx; ury = by;
-		} else {
-		    llx = min(llx, sx); lly = min(lly, sy);
-		    urx = max(urx, bx); ury = max(ury, by);
+		for (e = compound->ellipses; e != NULL; e = e->next) {
+			if (adjust_boundingbox && !depth_filter(e->depth))
+				continue;
+			ellipse_bound(e, &sx, &sy, &bx, &by);
+			if (first) {
+				first = 0;
+				llx = sx;
+				lly = sy;
+				urx = bx;
+				ury = by;
+			} else {
+				llx = min(llx, sx);
+				lly = min(lly, sy);
+				urx = max(urx, bx);
+				ury = max(ury, by);
+			}
 		}
-	    }
 
-	    for (l = compound->lines; l != NULL; l = l->next) {
-		if (adjust_boundingbox && !depth_filter(l->depth))
-		  continue;
-		line_bound(l, &sx, &sy, &bx, &by);
-		/* pictures have no line thickness */
-		if (l->type == T_PIC_BOX)
-		    half_wd = 0;
-		else
-		    half_wd = ceil((double)(l->thickness+1) / sqrt(2.0));
-		/* leave space for corners, better approach needs much more math! */
-		if (first) {
-		    first = 0;
-		    llx = sx - half_wd; lly = sy - half_wd;
-		    urx = bx + half_wd; ury = by + half_wd;
-		} else {
-		    llx = min(llx, sx - half_wd); lly = min(lly, sy - half_wd);
-		    urx = max(urx, bx + half_wd); ury = max(ury, by + half_wd);
+		for (l = compound->lines; l != NULL; l = l->next) {
+			if (adjust_boundingbox && !depth_filter(l->depth))
+				continue;
+			line_bound(l, &sx, &sy, &bx, &by);
+			/* pictures have no line thickness */
+			if (l->type == T_PIC_BOX)
+				half_wd = 0;
+			else
+				half_wd = ceil((double)(l->thickness+1) /
+						sqrt(2.0));
+			/* leave space for corners, better approach needs
+			   much more math! */
+			if (first) {
+				first = 0;
+				llx = sx - half_wd;
+				lly = sy - half_wd;
+				urx = bx + half_wd;
+				ury = by + half_wd;
+			} else {
+				llx = min(llx, sx - half_wd);
+				lly = min(lly, sy - half_wd);
+				urx = max(urx, bx + half_wd);
+				ury = max(ury, by + half_wd);
+			}
 		}
-	    }
 
-	    for (s = compound->splines; s != NULL; s = s->next) {
-		if (adjust_boundingbox && !depth_filter(s->depth))
-		  continue;
-		spline_bound(s, &sx, &sy, &bx, &by);
-		half_wd = (s->thickness+1) / 2;
-		if (first) {
-		    first = 0;
-		    llx = sx - half_wd; lly = sy - half_wd;
-		    urx = bx + half_wd; ury = by + half_wd;
-		} else {
-		    llx = min(llx, sx - half_wd); lly = min(lly, sy - half_wd);
-		    urx = max(urx, bx + half_wd); ury = max(ury, by + half_wd);
+		for (s = compound->splines; s != NULL; s = s->next) {
+			if (adjust_boundingbox && !depth_filter(s->depth))
+				continue;
+			spline_bound(s, &sx, &sy, &bx, &by);
+			half_wd = (s->thickness+1) / 2;
+			if (first) {
+				first = 0;
+				llx = sx - half_wd; lly = sy - half_wd;
+				urx = bx + half_wd; ury = by + half_wd;
+			} else {
+				llx = min(llx, sx - half_wd);
+				lly = min(lly, sy - half_wd);
+				urx = max(urx, bx + half_wd);
+				ury = max(ury, by + half_wd);
+			}
 		}
-	    }
 
-	    for (t = compound->texts; t != NULL; t = t->next) {
-		if (adjust_boundingbox && !depth_filter(t->depth))
-		  continue;
-		text_bound(t, &sx, &sy, &bx, &by, include);
-		if (first) {
-		    first = 0;
-		    llx = sx; lly = sy;
-		    urx = bx; ury = by;
-		} else {
-		    llx = min(llx, sx); lly = min(lly, sy);
-		    urx = max(urx, bx); ury = max(ury, by);
+		for (t = compound->texts; t != NULL; t = t->next) {
+			if (adjust_boundingbox && !depth_filter(t->depth))
+				continue;
+			text_bound(t, &sx, &sy, &bx, &by, include);
+			if (first) {
+				first = 0;
+				llx = sx; lly = sy;
+				urx = bx; ury = by;
+			} else {
+				llx = min(llx, sx); lly = min(lly, sy);
+				urx = max(urx, bx); ury = max(ury, by);
+			}
 		}
-	    }
-	    compound = compound->next;
+		compound = compound->next;
 	}
 
 	*xmin = llx; *ymin = lly;
@@ -451,7 +472,7 @@ line_bound(F_line *l, int *xmin, int *ymin, int *xmax, int *ymax)
 	/* now add in the arrow (if any) boundaries but
 	   only if the line has two or more points */
 	if (l->points->next)
-	    arrow_bound(OBJ_POLYLINE, l, xmin, ymin, xmax, ymax);
+		arrow_bound(OBJ_POLYLINE, l, xmin, ymin, xmax, ymax);
 }
 
 static void
@@ -468,30 +489,30 @@ int_spline_bound(F_spline *s, int *xmin, int *ymin, int *xmax, int *ymax)
 	sy = by = p1->y;
 	cp1 = s->controls;
 	for (p2 = p1->next, cp2 = cp1->next; p2 != NULL;
-		p1 = p2, cp1 = cp2, p2 = p2->next, cp2 = cp2->next) {
-	    x0 = p1->x; y0 = p1->y;
-	    x1 = cp1->rx; y1 = cp1->ry;
-	    x2 = cp2->lx; y2 = cp2->ly;
-	    x3 = p2->x; y3 = p2->y;
-	    tx = half(x1, x2); ty = half(y1, y2);
-	    sx1 = half(x0, x1); sy1 = half(y0, y1);
-	    sx2 = half(sx1, tx); sy2 = half(sy1, ty);
-	    tx2 = half(x2, x3); ty2 = half(y2, y3);
-	    tx1 = half(tx2, tx); ty1 = half(ty2, ty);
+			p1 = p2, cp1 = cp2, p2 = p2->next, cp2 = cp2->next) {
+		x0 = p1->x; y0 = p1->y;
+		x1 = cp1->rx; y1 = cp1->ry;
+		x2 = cp2->lx; y2 = cp2->ly;
+		x3 = p2->x; y3 = p2->y;
+		tx = half(x1, x2); ty = half(y1, y2);
+		sx1 = half(x0, x1); sy1 = half(y0, y1);
+		sx2 = half(sx1, tx); sy2 = half(sy1, ty);
+		tx2 = half(x2, x3); ty2 = half(y2, y3);
+		tx1 = half(tx2, tx); ty1 = half(ty2, ty);
 
-	    sx = min(x0, sx); sy = min(y0, sy);
-	    sx = min(sx1, sx); sy = min(sy1, sy);
-	    sx = min(sx2, sx); sy = min(sy2, sy);
-	    sx = min(tx1, sx); sy = min(ty1, sy);
-	    sx = min(tx2, sx); sy = min(ty2, sy);
-	    sx = min(x3, sx); sy = min(y3, sy);
+		sx = min(x0, sx); sy = min(y0, sy);
+		sx = min(sx1, sx); sy = min(sy1, sy);
+		sx = min(sx2, sx); sy = min(sy2, sy);
+		sx = min(tx1, sx); sy = min(ty1, sy);
+		sx = min(tx2, sx); sy = min(ty2, sy);
+		sx = min(x3, sx); sy = min(y3, sy);
 
-	    bx = max(x0, bx); by = max(y0, by);
-	    bx = max(sx1, bx); by = max(sy1, by);
-	    bx = max(sx2, bx); by = max(sy2, by);
-	    bx = max(tx1, bx); by = max(ty1, by);
-	    bx = max(tx2, bx); by = max(ty2, by);
-	    bx = max(x3, bx); by = max(y3, by);
+		bx = max(x0, bx); by = max(y0, by);
+		bx = max(sx1, bx); by = max(sy1, by);
+		bx = max(sx2, bx); by = max(sy2, by);
+		bx = max(tx1, bx); by = max(ty1, by);
+		bx = max(tx2, bx); by = max(ty2, by);
+		bx = max(x3, bx); by = max(y3, by);
 	}
 	*xmin = round(sx);
 	*ymin = round(sy);
@@ -514,35 +535,35 @@ normal_spline_bound(F_spline *s, int *xmin, int *ymin, int *xmax, int *ymax)
 	cx1 = (x1 + x2) / 2.0;   cy1 = (y1 + y2) / 2.0;
 	cx2 = (cx1 + x2) / 2.0;  cy2 = (cy1 + y2) / 2.0;
 	if (closed_spline(s)) {
-	    x1 = (cx1 + x1) / 2.0;
-	    y1 = (cy1 + y1) / 2.0;
+		x1 = (cx1 + x1) / 2.0;
+		y1 = (cy1 + y1) / 2.0;
 	}
 	sx = min(x1, cx2); sy = min(y1, cy2);
 	bx = max(x1, cx2); by = max(y1, cy2);
 
 	for (p = p->next; p != NULL; p = p->next) {
-	    x1 = x2;  y1 = y2;
-	    x2 = p->x;  y2 = p->y;
-	    cx4 = (x1 + x2) / 2.0; cy4 = (y1 + y2) / 2.0;
-	    cx3 = (x1 + cx4) / 2.0; cy3 = (y1 + cy4) / 2.0;
-	    cx2 = (cx4 + x2) / 2.0;  cy2 = (cy4 + y2) / 2.0;
+		x1 = x2;  y1 = y2;
+		x2 = p->x;  y2 = p->y;
+		cx4 = (x1 + x2) / 2.0; cy4 = (y1 + y2) / 2.0;
+		cx3 = (x1 + cx4) / 2.0; cy3 = (y1 + cy4) / 2.0;
+		cx2 = (cx4 + x2) / 2.0;  cy2 = (cy4 + y2) / 2.0;
 
-	    px = min(cx2, cx3); py = min(cy2, cy3);
-	    qx = max(cx2, cx3); qy = max(cy2, cy3);
+		px = min(cx2, cx3); py = min(cy2, cy3);
+		qx = max(cx2, cx3); qy = max(cy2, cy3);
 
-	    sx = min(sx, px); sy = min(sy, py);
-	    bx = max(bx, qx); by = max(by, qy);
+		sx = min(sx, px); sy = min(sy, py);
+		bx = max(bx, qx); by = max(by, qy);
 	}
 	if (closed_spline(s)) {
-	    *xmin = floor(sx );
-	    *ymin = floor(sy );
-	    *xmax = ceil (bx );
-	    *ymax = ceil (by );
+		*xmin = floor(sx );
+		*ymin = floor(sy );
+		*xmax = ceil (bx );
+		*ymax = ceil (by );
 	} else {
-	    *xmin = floor(min(sx, x2) );
-	    *ymin = floor(min(sy, y2) );
-	    *xmax = ceil (max(bx, x2) );
-	    *ymax = ceil (max(by, y2) );
+		*xmin = floor(min(sx, x2) );
+		*ymin = floor(min(sy, y2) );
+		*xmax = ceil (max(bx, x2) );
+		*ymax = ceil (max(by, y2) );
 	}
 }
 
@@ -550,11 +571,11 @@ void
 spline_bound(F_spline *s, int *xmin, int *ymin, int *xmax, int *ymax)
 {
 	if (int_spline(s)) {
-	    int_spline_bound(s, xmin, ymin, xmax, ymax);
-	    }
+		int_spline_bound(s, xmin, ymin, xmax, ymax);
+	}
 	else {
-	    normal_spline_bound(s, xmin, ymin, xmax, ymax);
-	    }
+		normal_spline_bound(s, xmin, ymin, xmax, ymax);
+	}
 	/* now do any arrows */
 	arrow_bound(OBJ_SPLINE, (F_line *)s, xmin, ymin, xmax, ymax);
 }
@@ -580,92 +601,96 @@ text_bound(F_text *t, int *xmin, int *ymin, int *xmax, int *ymax, int inc_text)
 
 	bool	include;
 
-    /* include text only:
-     * 1. if inc_text is true AND
-     * 2. not special OR is special and contains either a "$" (inline equation) or backslash "\"
-     * */
-    include = (inc_text &&
-		((t->flags & SPECIAL_TEXT)==0 ||
-		 (strchr(t->cstring,'\\')==0 && strchr(t->cstring,'$')==0)));
-    /* look for descenders in string (this is a kludge - next version
-       of xfig should include ascent/descent in text structure */
-    descend = (strchr(t->cstring,'g') || strchr(t->cstring,'j') ||
-		  strchr(t->cstring,'p') || strchr(t->cstring,'q') ||
-		  strchr(t->cstring,'y') || strchr(t->cstring,'$') ||
-		  strchr(t->cstring,'(') || strchr(t->cstring,')') ||
-		  strchr(t->cstring,'{') || strchr(t->cstring,'}') ||
-		  strchr(t->cstring,'[') || strchr(t->cstring,']') ||
-		  strchr(t->cstring,',') || strchr(t->cstring,';'));
+	/*
+	 * include text only:
+	 *  1. if inc_text is true AND
+	 *  2. not special OR is special and contains either
+	 *     a "$" (inline equation) or backslash "\"
+	 */
+	include = (inc_text &&
+			((t->flags & SPECIAL_TEXT) == 0 ||
+				(!strchr(t->cstring,'\\') &&
+				 !strchr(t->cstring,'$'))));
+	/* look for descenders in string (this is a kludge - next version
+	   of xfig should include ascent/descent in text structure */
+	descend = (strchr(t->cstring,'g') || strchr(t->cstring,'j') ||
+			strchr(t->cstring,'p') || strchr(t->cstring,'q') ||
+			strchr(t->cstring,'y') || strchr(t->cstring,'$') ||
+			strchr(t->cstring,'(') || strchr(t->cstring,')') ||
+			strchr(t->cstring,'{') || strchr(t->cstring,'}') ||
+			strchr(t->cstring,'[') || strchr(t->cstring,']') ||
+			strchr(t->cstring,',') || strchr(t->cstring,';') ||
+			strchr(t->cstring,'_'));
 
-    /* check if Symbol font with any descenders */
-    if (!descend && psfont_text(t) && t->font == 32)
-	descend =
-		(strchr(t->cstring,'b')		/* beta  */ ||
-		 strchr(t->cstring,'c')		/* chi   */ ||
-		 strchr(t->cstring,'f')		/* phi   */ ||
-		 strchr(t->cstring,'g')		/* gamma */ ||
-		 strchr(t->cstring,'h')		/* eta   */ ||
-		 strchr(t->cstring,'j')		/* phi1  */ ||
-		 strchr(t->cstring,'m')		/* mu    */ ||
-		 strchr(t->cstring,'r')		/* rho   */ ||
-		 strchr(t->cstring,'x')		/* xi    */ ||
-		 strchr(t->cstring,'y')		/* psi   */ ||
-		 strchr(t->cstring,'z')		/* zeta  */ ||
-		 strchr(t->cstring,'C'+'\200')	/* weierstrass    */ ||
-		 strchr(t->cstring,'J'+'\200')	/* reflexsuperset */ ||
-		 strchr(t->cstring,'M'+'\200')	/* reflexsubset   */ ||
-		 strchr(t->cstring,'U'+'\200')	/* product        */ ||
-		 strchr(t->cstring,'a'+'\200')	/* angleleft      */ ||
-		 strchr(t->cstring,'e'+'\200')	/* summation      */ ||
-		 strchr(t->cstring,'f'+'\200')	/* parenlefttp    */ ||
-		 strchr(t->cstring,'h'+'\200')	/* parenleftbt    */ ||
-		 strchr(t->cstring,'q'+'\200')	/* angleright     */ ||
-		 strchr(t->cstring,'r'+'\200')	/* integral       */ ||
-		 strchr(t->cstring,'v'+'\200')	/* parenrighttp   */ ||
-		 strchr(t->cstring,'x'+'\200')	/* parenrightbt   */ ||
-		 strchr(t->cstring,'&'+'\200')); /* florin        */
+	/* check if Symbol font with any descenders */
+	if (!descend && psfont_text(t) && t->font == 32)
+		descend =
+			(strchr(t->cstring,'b')		/* beta  */ ||
+			 strchr(t->cstring,'c')		/* chi   */ ||
+			 strchr(t->cstring,'f')		/* phi   */ ||
+			 strchr(t->cstring,'g')		/* gamma */ ||
+			 strchr(t->cstring,'h')		/* eta   */ ||
+			 strchr(t->cstring,'j')		/* phi1  */ ||
+			 strchr(t->cstring,'m')		/* mu    */ ||
+			 strchr(t->cstring,'r')		/* rho   */ ||
+			 strchr(t->cstring,'x')		/* xi    */ ||
+			 strchr(t->cstring,'y')		/* psi   */ ||
+			 strchr(t->cstring,'z')		/* zeta  */ ||
+			 strchr(t->cstring,'C'+'\200')	/* weierstrass    */ ||
+			 strchr(t->cstring,'J'+'\200')	/* reflexsuperset */ ||
+			 strchr(t->cstring,'M'+'\200')	/* reflexsubset   */ ||
+			 strchr(t->cstring,'U'+'\200')	/* product        */ ||
+			 strchr(t->cstring,'a'+'\200')	/* angleleft      */ ||
+			 strchr(t->cstring,'e'+'\200')	/* summation      */ ||
+			 strchr(t->cstring,'f'+'\200')	/* parenlefttp    */ ||
+			 strchr(t->cstring,'h'+'\200')	/* parenleftbt    */ ||
+			 strchr(t->cstring,'q'+'\200')	/* angleright     */ ||
+			 strchr(t->cstring,'r'+'\200')	/* integral       */ ||
+			 strchr(t->cstring,'v'+'\200')	/* parenrighttp   */ ||
+			 strchr(t->cstring,'x'+'\200')	/* parenrightbt   */ ||
+			 strchr(t->cstring,'&'+'\200')); /* florin        */
 
-    /* characters have some extent downside */
-    if (t->type == T_CENTER_JUSTIFIED) {
-	dx1 = (include?  (t->length/1.95) : 0.0);	dy1 =  0.0;
-	dx2 = (include? -(t->length/1.95) : 0.0);	dy2 =  0.0;
-	dx3 = (include?  (t->length/1.95) : 0.0);	dy3 = -t->height;
-	dx4 = (include? -(t->length/1.95) : 0.0);	dy4 = -t->height;
-    } else if (t->type == T_RIGHT_JUSTIFIED) {
-	dx1 = 0.0;					dy1 =  0.0;
-	dx2 = (include? -t->length*1.0256 : 0.0);	dy2 =  0.0;
-	dx3 = 0.0;					dy3 = -t->height;
-	dx4 = (include? -t->length*1.0256 : 0.0);	dy4 = -t->height;
-    } else {
-	dx1 = (include ? t->length*1.0256 : 0.0);	dy1 =  0.0;
-	dx2 = 0.0;					dy2 =  0.0;
-	dx3 = (include ? t->length*1.0256 : 0.0);	dy3 = -t->height;
-	dx4 = 0.0;					dy4 = -t->height;
-    }
-    if (descend) {
-	dy1 = 0.3*t->height;
-	dy2 = 0.3*t->height;
-	dy3 = -0.8*t->height;
-	dy4 = -0.8*t->height;
-    }
+	/* characters have some extent downside */
+	if (t->type == T_CENTER_JUSTIFIED) {
+		dx1 = (include?  (t->length/1.95) : 0.0);    dy1 =  0.0;
+		dx2 = (include? -(t->length/1.95) : 0.0);    dy2 =  0.0;
+		dx3 = (include?  (t->length/1.95) : 0.0);    dy3 = -t->height;
+		dx4 = (include? -(t->length/1.95) : 0.0);    dy4 = -t->height;
+	} else if (t->type == T_RIGHT_JUSTIFIED) {
+		dx1 = 0.0;				     dy1 =  0.0;
+		dx2 = (include? -t->length*1.0256 : 0.0);    dy2 =  0.0;
+		dx3 = 0.0;				     dy3 = -t->height;
+		dx4 = (include? -t->length*1.0256 : 0.0);    dy4 = -t->height;
+	} else {
+		dx1 = (include ? t->length*1.0256 : 0.0);    dy1 =  0.0;
+		dx2 = 0.0;				     dy2 =  0.0;
+		dx3 = (include ? t->length*1.0256 : 0.0);    dy3 = -t->height;
+		dx4 = 0.0;				     dy4 = -t->height;
+	}
+	if (descend) {
+		dy1 = 0.3*t->height;
+		dy2 = 0.3*t->height;
+		dy3 = -0.8*t->height;
+		dy4 = -0.8*t->height;
+	}
 
-    *xmax= t->base_x +
-           max( max( rot_x(dx1,dy1,t->angle), rot_x(dx2,dy2,t->angle) ),
-		max( rot_x(dx3,dy3,t->angle), rot_x(dx4,dy4,t->angle) ) )
-	   + THICK_SCALE;
-    *ymax= t->base_y +
-           max( max( rot_y(dx1,dy1,t->angle), rot_y(dx2,dy2,t->angle) ),
-		max( rot_y(dx3,dy3,t->angle), rot_y(dx4,dy4,t->angle) ) )
-	   + THICK_SCALE;
+	*xmax = t->base_x +
+		max(max(rot_x(dx1,dy1,t->angle), rot_x(dx2,dy2,t->angle)),
+			max(rot_x(dx3,dy3,t->angle), rot_x(dx4,dy4,t->angle))) +
+		THICK_SCALE;
+	*ymax = t->base_y +
+		max(max(rot_y(dx1,dy1,t->angle), rot_y(dx2,dy2,t->angle)),
+			max(rot_y(dx3,dy3,t->angle), rot_y(dx4,dy4,t->angle))) +
+		THICK_SCALE;
 
-    *xmin= t->base_x +
-	   min( min( rot_x(dx1,dy1,t->angle), rot_x(dx2,dy2,t->angle) ),
-		min( rot_x(dx3,dy3,t->angle), rot_x(dx4,dy4,t->angle) ) )
-	   - THICK_SCALE;
-    *ymin= t->base_y +
-           min( min( rot_y(dx1,dy1,t->angle), rot_y(dx2,dy2,t->angle) ),
-		min( rot_y(dx3,dy3,t->angle), rot_y(dx4,dy4,t->angle) ) )
-	   - THICK_SCALE;
+	*xmin = t->base_x +
+		min(min(rot_x(dx1,dy1,t->angle), rot_x(dx2,dy2,t->angle)),
+			min(rot_x(dx3,dy3,t->angle), rot_x(dx4,dy4,t->angle))) -
+		THICK_SCALE;
+	*ymin = t->base_y +
+		min(min(rot_y(dx1,dy1,t->angle), rot_y(dx2,dy2,t->angle)),
+			min(rot_y(dx3,dy3,t->angle), rot_y(dx4,dy4,t->angle))) -
+		THICK_SCALE;
 }
 
 static void
@@ -676,9 +701,9 @@ points_bound(F_point *points, int *xmin, int *ymin, int *xmax, int *ymax)
 
 	bx = sx = points->x; by = sy = points->y;
 	for (p = points->next; p != NULL; p = p->next) {
-	    sx = min(sx, p->x); sy = min(sy, p->y);
-	    bx = max(bx, p->x); by = max(by, p->y);
-	    }
+		sx = min(sx, p->x); sy = min(sy, p->y);
+		bx = max(bx, p->x); by = max(by, p->y);
+	}
 	*xmin = sx; *ymin = sy;
 	*xmax = bx; *ymax = by;
 }
@@ -721,67 +746,75 @@ arrow_bound(int objtype, F_line *obj, int *xmin, int *ymin, int *xmax,
 	F_pos	arrowpts[50], arrowdumpts[50];
 
 	if (obj->for_arrow) {
-	    if (objtype == OBJ_ARC) {
-		a = (F_arc *) obj;
-		compute_arcarrow_angle(a->center.x, a->center.y,
-			    (double)a->point[2].x, (double)a->point[2].y,
-			    a->direction, a->for_arrow, &p1x, &p1y);
-		p2x = a->point[2].x;	/* forward tip */
-		p2y = a->point[2].y;
-	    } else {
-		/* this doesn't work very well for a spline with few points
-		    and lots of curvature */
-		/* locate last point (forward tip) and next-to-last point */
-		for (p = obj->points; p->next; p = p->next)
-		    q = p;
-		p1x = q->x;
-		p1y = q->y;
-		p2x = p->x;
-		p2y = p->y;
-	    }
-	    calc_arrow(p1x, p1y, p2x, p2y, obj->thickness, obj->for_arrow,
+		if (objtype == OBJ_ARC) {
+			a = (F_arc *) obj;
+			compute_arcarrow_angle(a->center.x, a->center.y,
+					(double)a->point[2].x,
+					(double)a->point[2].y,
+					a->direction, a->for_arrow, &p1x, &p1y);
+			p2x = a->point[2].x;	/* forward tip */
+			p2y = a->point[2].y;
+		} else if (objtype == OBJ_POLYLINE) {
+			p1x = obj->last[1].x;
+			p1y = obj->last[1].y;
+			p2x = obj->last[0].x;
+			p2y = obj->last[0].y;
+		} else { /* objype == OBJ_SPLINE */
+			/* this doesn't work very well for a spline with few
+			   points and lots of curvature */
+
+			/* locate last point (forward tip) and
+			   next-to-last point */
+			for (p = obj->points; p->next; p = p->next)
+				q = p;
+			p1x = q->x;
+			p1y = q->y;
+			p2x = p->x;
+			p2y = p->y;
+		}
+		calc_arrow(p1x, p1y, p2x, p2y, obj->thickness, obj->for_arrow,
 			arrowpts, &npts, arrowdumpts, &dum, arrowdumpts, &dum);
-	    fxmin=fymin=10000000;
-	    fxmax=fymax=-10000000;
-	    for (i=0; i<npts; i++) {
-		fxmin = MIN(fxmin, arrowpts[i].x);
-		fymin = MIN(fymin, arrowpts[i].y);
-		fxmax = MAX(fxmax, arrowpts[i].x);
-		fymax = MAX(fymax, arrowpts[i].y);
-	    }
-	    *xmin = MIN(*xmin, fxmin);
-	    *xmax = MAX(*xmax, fxmax);
-	    *ymin = MIN(*ymin, fymin);
-	    *ymax = MAX(*ymax, fymax);
+		fxmin=fymin=10000000;
+		fxmax=fymax=-10000000;
+		for (i=0; i<npts; i++) {
+			fxmin = MIN(fxmin, arrowpts[i].x);
+			fymin = MIN(fymin, arrowpts[i].y);
+			fxmax = MAX(fxmax, arrowpts[i].x);
+			fymax = MAX(fymax, arrowpts[i].y);
+		}
+		*xmin = MIN(*xmin, fxmin);
+		*xmax = MAX(*xmax, fxmax);
+		*ymin = MIN(*ymin, fymin);
+		*ymax = MAX(*ymax, fymax);
 	}
 	if (obj->back_arrow) {
-	    if (objtype == OBJ_ARC) {
-		a = (F_arc *) obj;
-		compute_arcarrow_angle(a->center.x, a->center.y,
-			    (double) a->point[0].x, (double) a->point[0].y,
-			    a->direction ^ 1, a->back_arrow, &p1x, &p1y);
-		p2x = a->point[0].x;	/* backward tip */
-		p2y = a->point[0].y;
-	    } else {
-		p1x = obj->points->next->x;	/* second point */
-		p1y = obj->points->next->y;
-		p2x = obj->points->x;	/* first point (forward tip) */
-		p2y = obj->points->y;
-	    }
-	    calc_arrow(p1x, p1y, p2x, p2y, obj->thickness, obj->back_arrow,
+		if (objtype == OBJ_ARC) {
+			a = (F_arc *) obj;
+			compute_arcarrow_angle(a->center.x, a->center.y,
+				(double) a->point[0].x, (double) a->point[0].y,
+				a->direction ^ 1, a->back_arrow, &p1x, &p1y);
+			p2x = a->point[0].x;	/* backward tip */
+			p2y = a->point[0].y;
+		} else {
+			p1x = obj->points->next->x;	/* second point */
+			p1y = obj->points->next->y;
+			p2x = obj->points->x;	/* first point (forward tip) */
+			p2y = obj->points->y;
+		}
+		calc_arrow(p1x, p1y, p2x, p2y, obj->thickness, obj->back_arrow,
 			arrowpts, &npts, arrowdumpts, &dum, arrowdumpts, &dum);
-	    bxmin=bymin=10000000;
-	    bxmax=bymax=-10000000;
-	    for (i=0; i<npts; i++) {
-		bxmin = MIN(bxmin, arrowpts[i].x);
-		bymin = MIN(bymin, arrowpts[i].y);
-		bxmax = MAX(bxmax, arrowpts[i].x);
-		bymax = MAX(bymax, arrowpts[i].y);
-	    }
-	    *xmin = MIN(*xmin, bxmin);
-	    *xmax = MAX(*xmax, bxmax);
-	    *ymin = MIN(*ymin, bymin);
-	    *ymax = MAX(*ymax, bymax);
+		bxmin=bymin=10000000;
+		bxmax=bymax=-10000000;
+		for (i=0; i<npts; i++) {
+			bxmin = MIN(bxmin, arrowpts[i].x);
+			bymin = MIN(bymin, arrowpts[i].y);
+			bxmax = MAX(bxmax, arrowpts[i].x);
+			bymax = MAX(bymax, arrowpts[i].y);
+		}
+		*xmin = MIN(*xmin, bxmin);
+		*xmax = MAX(*xmax, bxmax);
+		*ymin = MIN(*ymin, bymin);
+		*ymax = MAX(*ymax, bymax);
 	}
 }
 
@@ -864,7 +897,7 @@ calc_arrow(int x1, int y1, int x2, int y2, int linethick, F_arrow *arrow,
 	dx = x2 - x1;
 	dy = y1 - y2;
 	if (dx==0 && dy==0)
-	    return;
+		return;
 
 	/* lpt is the amount the arrowhead extends beyond the end of the
 	 * line because of the sharp point (miter join) */
@@ -872,13 +905,13 @@ calc_arrow(int x1, int y1, int x2, int y2, int linethick, F_arrow *arrow,
 	lpt = 0.0;
 	thk = THICKNESS(arrow->thickness);
 	if (tipmv > 0.0)
-	    lpt = thk * sqrt(wd*wd + tipmv*tipmv*len*len) / 2. / fabs(wd);
+		lpt = thk * sqrt(wd*wd + tipmv*tipmv*len*len) / 2. / fabs(wd);
 	else if (tipmv == 0.0)
-	    lpt = thk / 2.0;	/* types which have blunt end */
-			    /* (Don't adjust those with tipmv < 0) */
+		lpt = thk / 2.0;	/* types which have blunt end */
+	/* (Don't adjust those with tipmv < 0) */
 
 	/* alpha is the angle the line is relative to horizontal */
-	l = sqrt((double)dx * dx + dy * dy);
+	l = sqrt((double)dx * dx + (double)dy * dy);
 	sina = dy / l;
 	cosa = dx / l;
 
@@ -902,177 +935,180 @@ calc_arrow(int x1, int y1, int x2, int y2, int linethick, F_arrow *arrow,
 	maxy = -100000.0;
 
 	if (type == 5 || type == 6) {
-	    /*
-	     * CIRCLE and HALF-CIRCLE arrowheads
-	     *
-	     * We approximate circles with 40 points
-	     */
-	    double	maxx;
-	    double	fix_x, fix_y, xs, ys;
-	    double	angle, init_angle, radius, rads;
-	    int		phase;
+		/*
+		 * CIRCLE and HALF-CIRCLE arrowheads
+		 *
+		 * We approximate circles with 40 points
+		 */
+		double	maxx;
+		double	fix_x, fix_y, xs, ys;
+		double	angle, init_angle, radius, rads;
+		int		phase;
 
-	    /* use original dx, dy to get starting angle */
-	    init_angle = atan2(dy, dx);
-	    if (init_angle < 0.0) init_angle += M_2PI;
+		/* use original dx, dy to get starting angle */
+		init_angle = atan2(dy, dx);
+		if (init_angle < 0.0) init_angle += M_2PI;
 
-	    radius = len/2.0;
+		radius = len/2.0;
 
-	    /* (xs,ys) is a point the length of the arrowhead BACK from
-	     * the end of the shaft */
-	    /* for the half circle, use 0.0 */
-	    if (type == 5) {
-		l = len;
-		maxx = radius;
-	    } else {
-		l = 0.0;
-		maxx = 0.;
-	    }
-	    xs =  (xb - l) * cosa + yb * sina;
-	    ys = -(xb - l) * sina + yb * cosa;
+		/* (xs,ys) is a point the length of the arrowhead BACK from
+		 * the end of the shaft */
+		/* for the half circle, use 0.0 */
+		if (type == 5) {
+			l = len;
+			maxx = radius;
+		} else {
+			l = 0.0;
+			maxx = 0.;
+		}
+		xs =  (xb - l) * cosa + yb * sina;
+		ys = -(xb - l) * sina + yb * cosa;
 
-	    /* calc new (dx, dy) from moved endpoint to (xs, ys) */
-	    dx = mx - xs;
-	    dy = my - ys;
-	    fix_x = xs + (dx / 2.0);
-	    fix_y = ys + (dy / 2.0);
-	    /* choose number of points for circle */
-	    *npoints = np = 40;
+		/* calc new (dx, dy) from moved endpoint to (xs, ys) */
+		dx = mx - xs;
+		dy = my - ys;
+		fix_x = xs + (dx / 2.0);
+		fix_y = ys + (dy / 2.0);
+		/* choose number of points for circle */
+		*npoints = np = 40;
 
-	    if (type == 5) {
-		/* full circle */
-		init_angle = 5.0*M_PI_2 - init_angle;
-		rads = M_2PI;
-	    } else {
-		/* half circle */
-		init_angle = 3.0*M_PI_2 - init_angle;
-		rads = M_PI;
-	    }
+		if (type == 5) {
+			/* full circle */
+			init_angle = 5.0*M_PI_2 - init_angle;
+			rads = M_2PI;
+		} else {
+			/* half circle */
+			init_angle = 3.0*M_PI_2 - init_angle;
+			rads = M_PI;
+		}
 
-	    /* draw the half or full circle */
-	    for (i = 0; i < np; ++i) {
-		angle = init_angle - (rads * (double) i / (double) (np-1));
-		x = fix_x + radius * cos(angle);
-		points[i].x = floor(x + 0.5);
-		y = fix_y + radius * sin(angle);
-		points[i].y = floor(y + 0.5);
-	    }
+		/* draw the half or full circle */
+		for (i = 0; i < np; ++i) {
+			angle = init_angle - (rads * i / (double)(np - 1));
+			x = fix_x + radius * cos(angle);
+			points[i].x = floor(x + 0.5);
+			y = fix_y + radius * sin(angle);
+			points[i].y = floor(y + 0.5);
+		}
 
-	    /* set clipping to a box at least as large as the line thickness
-	     * or diameter of the circle, whichever is larger */
-	    /* 4 points in clip box */
-	    thickline = halfthick > radius;
+		/* set clipping to a box at least as large as the line thickness
+		 * or diameter of the circle, whichever is larger */
+		/* 4 points in clip box */
+		thickline = halfthick > radius;
 
-	    /* start at first point (half-circle), or half into the circle */
-	    if (type == 6)
-		i = 0;
-	    else
-		i =  np/2;
-	    for (phase = i; i < np; ++i) {
-		clippts[i - phase].x = points[i].x;
-		clippts[i - phase].y = points[i].y;
-	    }
-	    i -= phase;
-	    if (thickline) {
-		clippts[i].x = ROTXC(0, clipthick);
-		clippts[i].y = ROTYC(0, clipthick);
+		/* start at first point (half-circle) or half into the circle */
+		if (type == 6)
+			i = 0;
+		else
+			i =  np / 2;
+		for (phase = i; i < np; ++i) {
+			clippts[i - phase].x = points[i].x;
+			clippts[i - phase].y = points[i].y;
+		}
+		i -= phase;
+		if (thickline) {
+			clippts[i].x = ROTXC(0, clipthick);
+			clippts[i].y = ROTYC(0, clipthick);
+			++i;
+		}
+		clippts[i].x = ROTXC(maxx + clipthick, clipthick);
+		clippts[i].y = ROTYC(maxx + clipthick, clipthick);
 		++i;
-	    }
-	    clippts[i].x = ROTXC(maxx + clipthick, clipthick);
-	    clippts[i].y = ROTYC(maxx + clipthick, clipthick);
-	    ++i;
-	    clippts[i].x = ROTXC(maxx + clipthick, -clipthick);
-	    clippts[i].y = ROTYC(maxx + clipthick, -clipthick);
-	    ++i;
-	    if (thickline) {
-		clippts[i].x = ROTXC(0, -clipthick);
-		clippts[i].y = ROTYC(0, -clipthick);
+		clippts[i].x = ROTXC(maxx + clipthick, -clipthick);
+		clippts[i].y = ROTYC(maxx + clipthick, -clipthick);
 		++i;
-	    }
-	    *nclippts = i;
+		if (thickline) {
+			clippts[i].x = ROTXC(0, -clipthick);
+			clippts[i].y = ROTYC(0, -clipthick);
+			++i;
+		}
+		*nclippts = i;
 
 	} else {
-	    /*
-	     * ALL OTHER HEADS
-	     */
-	    double	offset;
+		/*
+		 * ALL OTHER HEADS
+		 */
+		double	offset;
 
-	    *npoints = arrow_shapes[indx].numpts;
-	    /* we'll shift the half arrowheads down by the difference of the
-	     * main line thickness and the arrowhead thickness to make it flush
-	     * with the main line */
-	    if (arrow_shapes[indx].half)
-		/* offset = (linethick - arrow->thickness)/2; */
-		offset = (line_thk - thk) / 2.;
-	    else
-		offset = 0.;
+		*npoints = arrow_shapes[indx].numpts;
+		/*
+		 * we'll shift the half arrowheads down by the difference of the
+		 * main line thickness and the arrowhead thickness to make it
+		 * flush with the main line
+		 */
+		if (arrow_shapes[indx].half)
+			/* offset = (linethick - arrow->thickness)/2; */
+			offset = (line_thk - thk) / 2.;
+		else
+			offset = 0.;
 
-	    /* fill the points array with the outline */
-	    for (i = 0; i < *npoints; ++i) {
-		x = arrow_shapes[indx].points[i].x * len;
-		y = arrow_shapes[indx].points[i].y * wd - offset;
-		miny = MIN(y, miny);
-		maxy = MAX(y, maxy);
-		points[i].x = ROTX(x,y);
-		points[i].y = ROTY(x,y);
-	    }
+		/* fill the points array with the outline */
+		for (i = 0; i < *npoints; ++i) {
+			x = arrow_shapes[indx].points[i].x * len;
+			y = arrow_shapes[indx].points[i].y * wd - offset;
+			miny = MIN(y, miny);
+			maxy = MAX(y, maxy);
+			points[i].x = ROTX(x,y);
+			points[i].y = ROTY(x,y);
+		}
 
-	    /* and the fill points array if there are fill points different
-	     * from the outline */
-	    *nfillpoints = arrow_shapes[indx].numfillpts;
-	    for (i = 0; i < *nfillpoints; ++i) {
-		x = arrow_shapes[indx].fillpoints[i].x * len;
-		y = arrow_shapes[indx].fillpoints[i].y * wd - offset;
-		miny = MIN(y, miny);
-		maxy = MAX(y, maxy);
-		fillpoints[i].x = ROTX(x,y);
-		fillpoints[i].y = ROTY(x,y);
-	    }
+		/* and the fill points array if there are fill points different
+		 * from the outline */
+		*nfillpoints = arrow_shapes[indx].numfillpts;
+		for (i = 0; i < *nfillpoints; ++i) {
+			x = arrow_shapes[indx].fillpoints[i].x * len;
+			y = arrow_shapes[indx].fillpoints[i].y * wd - offset;
+			miny = MIN(y, miny);
+			maxy = MAX(y, maxy);
+			fillpoints[i].x = ROTX(x,y);
+			fillpoints[i].y = ROTY(x,y);
+		}
 
-	    /* set clipping to the last points of the arrowhead and
-	     * the (enlarged) box surrounding it */
-	    *nclippts = 0;
-	    for (i = 0;
-		 i < arrow_shapes[indx].numpts - arrow_shapes[indx].startclip;
-		 ++i) {
-		clippts[i].x = points[i + arrow_shapes[indx].startclip].x;
-		clippts[i].y = points[i + arrow_shapes[indx].startclip].y;
-	    }
+		/* set clipping to the last points of the arrowhead and
+		 * the (enlarged) box surrounding it */
+		*nclippts = 0;
+		for (i = 0; i < arrow_shapes[indx].numpts -
+				arrow_shapes[indx].startclip; ++i) {
+			clippts[i].x =
+				points[i + arrow_shapes[indx].startclip].x;
+			clippts[i].y =
+				points[i + arrow_shapes[indx].startclip].y;
+		}
 
-	    /* now make the box around it at least as large as the line
-	     * thickness */
-	    /* start with last x, upper y */
-	    if (halfthick > maxy) {
-		x = arrow_shapes[indx].points[*npoints - 1].x * len;
-		clippts[i].x = ROTX(x, clipthick);
-		clippts[i].y = ROTY(x, clipthick);
+		/* now make the box around it at least as large as the line
+		 * thickness */
+		/* start with last x, upper y */
+		if (halfthick > maxy) {
+			x = arrow_shapes[indx].points[*npoints - 1].x * len;
+			clippts[i].x = ROTX(x, clipthick);
+			clippts[i].y = ROTY(x, clipthick);
+			++i;
+		}
+
+		/* x tip (always == 0), same y (note different offset in ROTX/Y2
+		   rotation) */
+		/* add halfthick in case the cap style is Round or Projecting */
+		clippts[i].x = ROTX2(clipthick, clipthick);
+		clippts[i].y = ROTY2(clipthick, clipthick);
 		++i;
-	    }
 
-	    /* x tip (always == 0), same y (note different offset in ROTX/Y2
-	     * rotation) */
-	    /* add halfthick in case the line cap style is Round or Projecting */
-	    clippts[i].x = ROTX2(clipthick, clipthick);
-	    clippts[i].y = ROTY2(clipthick, clipthick);
-	    ++i;
-
-	    /* x tip, upper y (note different offset in ROTX/Y2
-	     * rotation) */
-	    /* add halfthick in case the line cap style is Round or Projecting */
-	    clippts[i].x = ROTX2(clipthick, -clipthick);
-	    clippts[i].y = ROTY2(clipthick, -clipthick);
-	    ++i;
-
-	    if (-halfthick < miny) {
-		x = arrow_shapes[indx].points[arrow_shapes[indx].startclip].x *
-		    len;
-		clippts[i].x = ROTX(x, -clipthick);
-		clippts[i].y = ROTY(x, -clipthick);
+		/* x tip, upper y (note different offset in ROTX/Y2
+		   rotation) */
+		/* add halfthick in case the cap style is Round or Projecting */
+		clippts[i].x = ROTX2(clipthick, -clipthick);
+		clippts[i].y = ROTY2(clipthick, -clipthick);
 		++i;
-	    }
 
-	    /* set the number of points in the clip or bounds */
-	    *nclippts = i;
+		if (-halfthick < miny) {
+			x = arrow_shapes[indx].points[
+				arrow_shapes[indx].startclip].x * len;
+			clippts[i].x = ROTX(x, -clipthick);
+			clippts[i].y = ROTY(x, -clipthick);
+			++i;
+		}
+
+		/* set the number of points in the clip or bounds */
+		*nclippts = i;
 	}
 }
 
@@ -1104,15 +1140,15 @@ compute_arcarrow_angle(double x1, double y1, double x2, double y2,
 
 	/* secant would be too large or too small */
 	if (h > 2.0*r || h < 0.01*r) {
-	    arc_tangent_int(x1,y1,x2,y2,direction,x,y);
-	    return;
+		arc_tangent_int(x1,y1,x2,y2,direction,x,y);
+		return;
 	}
 
 	beta=atan2(dy,dx);
 	if (direction) {
-	    alpha = 2*asin(h/2.0/r);
+		alpha = 2*asin(h/2.0/r);
 	} else {
-	    alpha = -2*asin(h/2.0/r);
+		alpha = -2*asin(h/2.0/r);
 	}
 
 	*x=round(x1+r*cos(beta+alpha));
