@@ -1,6 +1,9 @@
 /*
  * Fig2dev: Translate Fig code to various Devices
- * Parts Copyright (c) 2015-2017 by Thomas Loimer
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2019 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -19,5 +22,10 @@
  *
  */
 
+#ifndef COLORS_H
+#define COLORS_H
+
 extern int	lookup_X_color(char *name, RGB *rgb);
-extern float	rgb2luminance (float r, float g, float b);
+extern double	rgb2luminance (double r, double g, double b);
+
+#endif /* COLORS_H */
