@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2020 by Thomas Loimer
+ * Parts Copyright (c) 2015-2022 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -25,8 +25,6 @@
  *	  adapted from:
  *
  *  genibmgl.c:	IBMGL driver for fig2dev
- *		  IBM 6180 Color Plotter with
- *		  IBM Graphics Enhancement Cartridge
  *   Author E. Robert Tisdale, University of California, 1/92
  *  (edwin@cs.ucla.edu)
  *
@@ -34,20 +32,50 @@
  *
  *  genpictex.c:	PiCTeX driver for fig2dev
  *   Author Micah Beck, Cornell University, 4/88
- *  Color, rotated text and ISO-chars added by Herbert Bauer 11/91
- *  PCL job control option added Brian V. Smith 1/2001
+ *
  */
 
+/*
+ * TODO
+ * insert Limits into the header,
+ *	$LIMMIN
+ *	10
+ *	-2		* min. x-coord (default mm)
+ *	20
+ *	-2		* min y-coord
+ *	9
+ *	$LIMMAX
+ *	10
+ *	2		* x-coord
+ *	20
+ *	2		* y-coord
+ *	9
+ *	$PLIMMIN	* paper limits ?
+ *	10
+ *	0		* x-coord
+ *	20
+ *	0		* y-coord
+ *	9
+ *	$PLIMMAX
+ *	10
+ *	210		* x-coord
+ *	20
+ *	297		* y-coord
+ */
+
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef	HAVE_STRINGS_H
+#include <strings.h>			/* strcasecmp */
+#endif
 #include <math.h>
 
 #include "fig2dev.h"	/* includes bool.h and object.h */
-//#include "object.h"
 #include "localmath.h"
 #include "messages.h"
 #include "pi.h"
@@ -59,37 +87,15 @@ static void set_style(int style, double length);
 #define PATTERNS	21
 #define DPR		180.0/M_PI	/* degrees/radian */
 #define DELTA		M_PI/36.0	/* radians */
-#define POINT_PER_INCH	72.27		/* points/inch */
-#define CMPP		254.0/7227.0	/* centimeters/point */
-#define UNITS_PER_INCH	1016.0		/* plotter units/inch */
-#define HEIGHT		7650.0		/* plotter units */
-#define ISO_A4		10900.0		/* plotter units */
-#define ANSI_A		10300.0		/* plotter units */
-#define SPEED_LIMIT	128.0		/* centimeters/second */
 
-#ifdef IBMGEC
-static int	ibmgec = true;
-#else
-static int	ibmgec = false;
-#endif
-
-static bool	reflected = false;
 static int	fonts = FONTS;
 static int	colors = COLORS;
 static int	patterns = PATTERNS;
 static int	line_style = SOLID_LINE;
-static int	fill_pattern = DEFAULT;
-static double	dash_length = DEFAULT;		/* in pixels */
-#ifdef A4
-static double	pagelength = ISO_A4/UNITS_PER_INCH;
-#else
-static double	pagelength = ANSI_A/UNITS_PER_INCH;
-#endif
-static double	pageheight = HEIGHT/UNITS_PER_INCH;
-static double	xl =  0.0;		/* inches */
-static double	yl =  0.0;		/* inches */
-static double	xu = 32.25;		/* inches */
-static double	yu = 32.25;		/* inches */
+static double	dash_length = DEFAULT;
+static double	figtodxf;		/* dxf (mm|in) per fig unit */
+					/* fig unit: usually 1200 / inch */
+static double	paperheight;
 
 static int	pen_number[] = { 1, 2, 3, 4, 5, 6, 7, 8, 1};
 static double	pen_thickness[] = {.3,.3,.3,.3,.3,.3,.3,.3,.3};
@@ -119,26 +125,18 @@ static double	high[] = {.8,.8,.8,.8,.8,.8,.8,.8,.8,.8,.8,.8,.8,.8,
 static void
 gendxf_option(char opt, char *optarg)
 {
-	// (void)	optarg;
 	FILE	*ffp;
 	int	font;
 	int	pattern;
 
 	switch (opt) {
 	case 'a':				/* paper size */
-#ifdef A4
-		pagelength = ANSI_A/UNITS_PER_INCH;
-#else
-		pagelength = ISO_A4/UNITS_PER_INCH;
-#endif
 		break;
 
-	case 'c':		/* Graphics Enhancement Cartridge emulation */
-		ibmgec = !ibmgec;
+	case 'c':
 		break;
 
 	case 'd':				/* position and window, inches*/
-		sscanf(optarg, "%lf,%lf,%lf,%lf", &xl,&yl,&xu,&yu);
 		break;
 
 	case 'f':				/* user's characters */
@@ -194,7 +192,6 @@ gendxf_option(char opt, char *optarg)
 		break;
 
 	case 'v':
-		reflected = true;		/* mirror image */
 		break;
 
 	default:
@@ -203,63 +200,52 @@ gendxf_option(char opt, char *optarg)
 	}
 }
 
-static double	cpi;		/* cent/inch */
-static double	cpp;		/* cent/pixel */
-static double	hcmpp = CMPP;	/* centimeter/point */
+static double	cm;		/* dxf unit (in|mm) per cm */
+static double	styletodxf;	/* dxf unit per in/80 (base line thickness) */
 
 void
 gendxf_start(F_compound *objects)
 {
-	(void)	objects;
+	(void)objects;
+	int	i = -1;
 
 	if (fabs(mag) < 1.0/2048.0){
 	    fprintf(stderr, "|mag| < 1/2048\n");
 	    exit(1);
 	    }
 
-	if (xl < xu)
-	    if (0.0 < xu)
-		if (xl < pagelength) {
-		    xl = (0.0 < xl) ? xl: 0.0;
-		    xu = (xu < pagelength) ? xu: pagelength;
-		    }
-		else {
-		    fprintf(stderr, "xll >= %.2f\n", pagelength);
-		    exit(1);
-		    }
-	    else {
-		fprintf(stderr, "xur <= 0.0\n");
-		exit(1);
-		}
-	else {
-	    fprintf(stderr, "xur <= xll\n");
-	    exit(1);
-	    }
+	if (metric) {
+		figtodxf = mag * 25.4 / ppi;	/* mm per fig unit */
+		cm = mag * 10;			/* mm per cm */
+		styletodxf = mag * 25.4 / 80.;	/* mm per in/80
+							(line thickness) */
+	} else {
+		figtodxf = mag / ppi;		/* inch per fig unit */
+		cm = mag / 2.54;		/* inch per cm */
+		styletodxf = mag / 80.;		/* in per in/80 */
+	}
 
-	if (yl < yu)
-	    if (0.0 < yu)
-		if (yl < pageheight) {
-		    yl = (0.0 < yl) ? yl: 0.0;
-		    yu = (yu < pageheight) ? yu: pageheight;
-		    }
-		else {
-		    fprintf(stderr, "yll >= %.2f\n", pageheight);
-		    exit(1);
-		    }
-	    else {
-		fprintf(stderr, "yur <= 0.0\n");
-		exit(1);
+	paperheight = paperdef[13].height;	/* paperdef[13] is A4 */
+	while (paperdef[++i].name) {
+		if (!strcasecmp(paperdef[i].name, papersize)) {
+			paperheight = paperdef[i].height;
+			break;
 		}
-	else {
-	    fprintf(stderr, "yur <= yll\n");
-	    exit(1);
-	    }
-
-	cpi = mag*100.0/sqrt((xu-xl)*(xu-xl) + (yu-yl)*(yu-yl));
-	cpp = cpi/ppi;
+	}
+	paperheight *= ppi / 72.;		/* points to fig units */
+#define YCOORD(y)	(paperheight - (y))
 
 	/* dxf start */
 	fprintf(tfp, "	0\nSECTION\n  2\nHEADER\n");
+
+	/* set the drawing unit */
+	fputs("  9\n$INSUNITS\n 70\n", tfp);
+	fputs(metric ? "  4\n" : "  1\n", tfp);	/* 1 - inches, 4 - mm */;
+
+	/* Dimension scale factor. If inches, librecad does not set this to 1 */
+	fputs("  9\n$DIMSCALE\n 40\n1.0\n", tfp);
+	fputs("  9\n$DIMLFAC\n 40\n1.0\n", tfp);
+
 	fprintf(tfp, "	0\nENDSEC\n");
 
 	fprintf(tfp, "999\n generated by gen_dxf v0.03\n");
@@ -321,12 +307,6 @@ gendxf_start(F_compound *objects)
 	fprintf(tfp, "	0\nSECTION\n  2\nBLOCKS\n");
 	fprintf(tfp, "	0\nENDSEC\n");
 	fprintf(tfp, "	0\nSECTION\n  2\nENTITIES\n");
-
-	if (!landscape) {			 /* portrait mode */
-	    if (reflected)			  /* upside-down text */
-		hcmpp	= -hcmpp;
-	}
-
 }
 
 /*	  draw arrow heading from (x1, y1) to (x2, y2) */
@@ -363,14 +343,14 @@ set_style(int style, double length)
 		case DASH_LINE:
 		    if (dash_length != length && length > 0.0) {
 			dash_length = length;
-			fprintf(tfp, "LT2,%.4f;\n", dash_length*2.0*cpp);
+			fprintf(tfp, "LT2,%.4f;\n", dash_length*2.0*styletodxf);
 			}
 		    break;
 
 		case DOTTED_LINE:
 		    if (dash_length != length && length > 0.0) {
 			dash_length = length;
-			fprintf(tfp, "LT1,%.4f;\n", dash_length*2.0*cpp);
+			fprintf(tfp, "LT1,%.4f;\n", dash_length*2.0*styletodxf);
 			}
 		    break;
 		}
@@ -385,7 +365,7 @@ set_style(int style, double length)
 		    if (dash_length != length && length > 0.0)
 			dash_length = length;
 		    if (dash_length > 0.0)
-			fprintf(tfp, "LT2,%.4f;\n", dash_length*2.0*cpp);
+			fprintf(tfp, "LT2,%.4f;\n", dash_length*2.0*styletodxf);
 		    else
 			fprintf(tfp, "LT2,-1.0;\n");
 		    break;
@@ -394,7 +374,7 @@ set_style(int style, double length)
 		    if (dash_length != length && length > 0.0)
 			dash_length = length;
 		    if (dash_length > 0.0)
-			fprintf(tfp, "LT1,%.4f;\n", dash_length*2.0*cpp);
+			fprintf(tfp, "LT1,%.4f;\n", dash_length*2.0*styletodxf);
 		    else
 			fprintf(tfp, "LT1,-1.0;\n");
 		    break;
@@ -402,192 +382,45 @@ set_style(int style, double length)
 	    }
 }
 
-/*
- * set_width - issue line width commands as appropriate
- *		  NOTE: for HP plotters we can't do anything
- */
-/*
-static void
-set_width(int w)
-{
-}
-*/
-
-static void
-fill_polygon(int pattern)
-{
-	if (0 < pattern && pattern < patterns) {
-	    int		       style;
-	    double	  length;
-
-	    if (fill_pattern != pattern) {
-		fill_pattern  = pattern;
-		fprintf(tfp, "FT%d,%.4f,%.4f;", fill_type[pattern],
-			fill_space[pattern],
-			reflected ? -fill_angle[pattern]: fill_angle[pattern]);
-		}
-	    /*	  save line style */
-	    style	  = line_style;
-	    length	   = dash_length;
-	    fprintf(tfp, "LT%d,%.4f;FP;\n",
-		    line_type[pattern], line_space[pattern]*cpi);
-	    /* restore line style */
-	    line_style	       = DEFAULT;
-	    dash_length		= DEFAULT;
-	    set_style(style, length);
-	    }
-}
-
-void
-dxf_arc(double sx, double sy, double cx, double cy, double theta, double delta)
-{
-  fprintf(tfp, "999\n !! found dxf_arc\n");
-  if (1 == 0) {
-	if (ibmgec)
-	    if (delta == M_PI/36.0)		   /* 5 degrees */
-		fprintf(tfp, "AA%.4f,%.4f,%.4f;",
-			cx, cy, theta*DPR);
-	    else
-		fprintf(tfp, "AA%.4f,%.4f,%.4f,%.4f;",
-			cx, cy, theta*DPR, delta*DPR);
-	else {
-	    double	  alpha;
-	    if (theta < 0.0)
-		delta = -fabs(delta);
-	    else
-		delta = fabs(delta);
-	    for (alpha = delta; fabs(alpha) < fabs(theta); alpha += delta) {
-		fprintf(tfp, "PA%.4f,%.4f;\n",
-			    cx + (sx - cx)*cos(alpha) - (sy - cy)*sin(alpha),
-			    cy + (sy - cy)*cos(alpha) + (sx - cx)*sin(alpha));
-		}
-	    fprintf(tfp, "PA%.4f,%.4f;\n",
-			cx + (sx - cx)*cos(theta) - (sy - cy)*sin(theta),
-			cy + (sy - cy)*cos(theta) + (sx - cx)*sin(theta));
-	    }
-  }
-}
-
 void
 gendxf_arc(F_arc *a)
 {
+	(void)a;
   fprintf(tfp, "999\n !! found gendxf_arc\n");
-  if (1 == 0) {
-	if (a->thickness != 0 ||
-		(ibmgec && 0 <= a->fill_style && a->fill_style < patterns)) {
-	    double	  x, y;
-	    double	  cx, cy, sx, sy, ex, ey;
-	    double	  dx1, dy1, dx2, dy2, theta;
-
-
-	    set_style(a->style, a->style_val);
-/*	    set_width(a->thickness);	*/
-
-	    cx = a->center.x/ppi;
-	    cy = a->center.y/ppi;
-	    sx = a->point[0].x/ppi;
-	    sy = a->point[0].y/ppi;
-	    ex = a->point[2].x/ppi;
-	    ey = a->point[2].y/ppi;
-
-	    dx1 = sx - cx;
-	    dy1 = sy - cy;
-	    dx2 = ex - cx;
-	    dy2 = ey - cy;
-
-	    theta = atan2(dy2, dx2) - atan2(dy1, dx1);
-	    if (a->direction) {
-		if (theta > 0.0)
-		    theta -= 2.0*M_PI;
-	    } else {
-		if (theta < 0.0)
-		    theta += 2.0*M_PI;
-	    }
-
-	    if (a->type == T_OPEN_ARC && a->thickness != 0 && a->back_arrow) {
-		arc_tangent(cx, cy, sx, sy, !a->direction, &x, &y);
-		draw_arrow_head(x, y, sx, sy);
-		}
-
-	    fprintf(tfp, "PA%.4f,%.4f;PM;PD;", sx, sy);
-	    dxf_arc(sx, sy, cx, cy, theta, DELTA);
-	    fprintf(tfp, "PU;PM2;\n");
-
-	    if (a->thickness != 0)
-		fprintf(tfp, "EP;\n");
-
-	    if (a->type == T_OPEN_ARC && a->thickness != 0 && a->for_arrow) {
-		arc_tangent(cx, cy, ex, ey, a->direction, &x, &y);
-		draw_arrow_head(x, y, ex, ey);
-		}
-
-	    if (0 < a->fill_style && a->fill_style < patterns)
-		fill_polygon(a->fill_style);
-	    }
-  }
 }
 
 void
 gendxf_ellipse(F_ellipse *e)
 {
-  /* This is a quick fix to use polylines rather than dxf ellipses */
-  /* This might be a compatibility option also in the future. */
-  if (e->thickness != 0 ||
-	    (ibmgec && 0 <= e->fill_style && e->fill_style < patterns)) {
-    int		       j;
-    double	  alpha;
-    double	  angle;
-    double	  delta;
-    double	  x0, y0;
-    double	  a,  b;
-    double	  x,  y;
-
-    set_style(e->style, e->style_val);
-/*    set_width(e->thickness);	*/
-
-    a	  = e->radiuses.x/ppi;
-    b	  = e->radiuses.y/ppi;
-    x0	  = e->center.x/ppi;
-    y0	  = e->center.y/ppi;
-    angle = -e->angle;
-    delta = -DELTA;
-
-    x = x0 + cos(angle)*a;
-    y = y0 + sin(angle)*a;
-
-    fprintf(tfp, "999\n !! found ellipse\n");
-
-    fprintf(tfp, "  0\nPOLYLINE\n");
-    fprintf(tfp, "  6\nSolid\n");
-    fprintf(tfp, "  8\n%3i\n",e->depth);
-    fprintf(tfp, " 66\n1\n");
-    fprintf(tfp, " 62\n%6i\n",e->pen_color);
-    fprintf(tfp, " 10\n0.0\n");
-    fprintf(tfp, " 20\n0.0\n");
-    fprintf(tfp, " 30\n0.0\n");
-    fprintf(tfp, " 70\n  1\n");
-
-    for (j = 1; j <= 72; j++) {
-      alpha = j*delta;
-      x     = x0 + cos(angle)*a*cos(alpha) - sin(angle)*b*sin(alpha);
-      y     = y0 + sin(angle)*a*cos(alpha) + cos(angle)*b*sin(alpha);
-      fprintf(tfp, "  0\nVERTEX\n");
-      fprintf(tfp, "  8\n%3i\n",e->depth);
-      fprintf(tfp, "  6\nSolid\n");
-      fprintf(tfp, " 10\n%f\n",x);
-      fprintf(tfp, " 20\n%f\n",y);
-      fprintf(tfp, " 30\n0.0\n");
-      fprintf(tfp, " 70\n  0\n");
-    }
-    fprintf(tfp, "  0\nSEQEND\n");
-  }
+	if (e->radiuses.x == e->radiuses.y) {
+		fputs("  0\nCIRCLE\n", tfp);
+		fputs("  6\nSolid\n", tfp);		/* line style */
+		fprintf(tfp, "  8\n%3d\n", e->depth);	/* layer name */
+		fputs(" 39\n 1\n", tfp);		/* thickness */
+		fprintf(tfp, " 10\n%.4g\n", e->center.x * figtodxf);
+		fprintf(tfp, " 20\n%.4f\n", YCOORD(e->center.y) * figtodxf);
+		fprintf(tfp, " 40\n%.4f\n", e->radiuses.x * figtodxf);
+	} else {
+		fputs("  0\nELLIPSE\n", tfp);
+		fputs("  6\nSolid\n", tfp);		/* line style */
+		fprintf(tfp, "  8\n%3d\n", e->depth);	/* layer name */
+		// fputs(" 39\n 1\n", tfp);		/* thickness */
+		fprintf(tfp, " 10\n%.4g\n", e->center.x * figtodxf);
+		fprintf(tfp, " 20\n%.4f\n", YCOORD(e->center.y) * figtodxf);
+		/* 11, 21: x and y-coordinate of endpoint of major axis,
+		   relative to center */
+		fprintf(tfp, " 11\n%.4f\n", e->radiuses.x * figtodxf);
+		fputs(" 21\n0.0\n", tfp);
+		fprintf(tfp, " 40\n %.6g\n",
+					(double)e->radiuses.y / e->radiuses.x);
+		fputs(" 41\n0.0\n 42\n6.28318\n", tfp);	/* start, end angle */
+	}
 }
 
 void
 gendxf_line(F_line *l)
 {
-  if (l->thickness != 0 || (ibmgec && 0 <= l->fill_style &&
-					l->fill_style < patterns))
+  if (l->thickness != 0 || (0 <= l->fill_style && l->fill_style < patterns))
   {
     F_point	   *p, *q, *r;
 
@@ -617,11 +450,12 @@ gendxf_line(F_line *l)
 	  fprintf(tfp, "  8\n%3i\n",l->depth);
 	  fprintf(tfp, "  6\nSolid\n");
 	  fprintf(tfp, " 62\n%6i\n",l->pen_color);
-	  fprintf(tfp, " 10\n%f\n",p->x/ppi);
-	  fprintf(tfp, " 20\n%f\n",p->y/ppi);
+	  fprintf(tfp, " 10\n%f\n",p->x * figtodxf);
+	  fprintf(tfp, " 20\n%f\n",YCOORD(p->y) * figtodxf);
 	} else {
 	  if (l->thickness != 0 && l->back_arrow)
-	      draw_arrow_head(q->x/ppi, q->y/ppi, p->x/ppi, p->y/ppi);
+	    draw_arrow_head(q->x * figtodxf, YCOORD(q->y) * figtodxf,
+			    p->x * figtodxf, YCOORD(p->y) * figtodxf);
 
 	  fprintf(tfp, "  0\nPOLYLINE\n");
 	  fprintf(tfp, "  6\nSolid\n");
@@ -640,10 +474,10 @@ gendxf_line(F_line *l)
 	  r = q = p;
 	  while (p != NULL) {
 	    fprintf(tfp, "  0\nVERTEX\n");
-	    fprintf(tfp, "  8\n%3i\n",l->depth);
+	    fprintf(tfp, "  8\n%3i\n", l->depth);
 	    fprintf(tfp, "  6\nSolid\n");
-	    fprintf(tfp, " 10\n%f\n",p->x/ppi);
-	    fprintf(tfp, " 20\n%f\n",p->y/ppi);
+	    fprintf(tfp, " 10\n%f\n", p->x * figtodxf);
+	    fprintf(tfp, " 20\n%f\n", YCOORD(p->y) * figtodxf);
 	    fprintf(tfp, " 30\n0.0\n");
 	    fprintf(tfp, " 70\n  0\n");
 	    r = q;
@@ -652,10 +486,8 @@ gendxf_line(F_line *l)
 	  }
 
 	  if (l->thickness != 0 && l->for_arrow)
-	    draw_arrow_head(r->x/ppi, r->y/ppi, q->x/ppi, q->y/ppi);
-
-/*	  if (0 < l->fill_style && l->fill_style < patterns)
-	    fill_polygon((int)l->fill_style, l->fill_color);	*/
+	    draw_arrow_head(r->x * figtodxf, YCOORD(r->y) * figtodxf,
+			    q->x * figtodxf, YCOORD(q->y) * figtodxf);
 
 	  fprintf(tfp, "  0\nSEQEND\n");
 	}
@@ -667,19 +499,19 @@ gendxf_line(F_line *l)
 	  double	dx, dy;
 
 	  llx	      = urx	   = p->x;
-	  lly	      = ury	   = p->y;
+	  lly	      = ury	   = YCOORD(p->y);
 	  while ((p = p->next) != NULL) {
 	    if (llx > p->x) llx = p->x;
 	    if (urx < p->x) urx = p->x;
-	    if (lly > p->y) lly = p->y;
-	    if (ury < p->y) ury = p->y;
+	    if (lly > p->y) lly = YCOORD(p->y);
+	    if (ury < p->y) ury = YCOORD(p->y);
 	  }
 
-	  x0 = llx/ppi;
-	  x1 = urx/ppi;
-	  dx = l->radius/ppi;
-	  y0 = ury/ppi;
-	  y1 = lly/ppi;
+	  x0 = llx * figtodxf;
+	  x1 = urx * figtodxf;
+	  dx = l->radius * figtodxf;
+	  y0 = ury * figtodxf;
+	  y1 = lly * figtodxf;
 	  dy = -dx;
 
 	  fprintf(tfp, "999\n !! found arc-box\n");
@@ -689,8 +521,8 @@ gendxf_line(F_line *l)
 	    fprintf(tfp, "  8\n%3i\n",l->depth);
 	    fprintf(tfp, "  6\nSolid\n");
 	    fprintf(tfp, " 62\n%6i\n",l->pen_color);
-	    fprintf(tfp, " 10\n%f\n",p->x/ppi);
-	    fprintf(tfp, " 20\n%f\n",p->y/ppi);
+	    fprintf(tfp, " 10\n%f\n",p->x * figtodxf);
+	    fprintf(tfp, " 20\n%f\n",YCOORD(p->y) * figtodxf);
 	  } else {
 	    fprintf(tfp, "  0\nLINE\n");
 	    fprintf(tfp, "  6\nSolid\n");
@@ -734,21 +566,6 @@ gendxf_line(F_line *l)
 
 
 	  }
-
-/*
- *	  fprintf(tfp, "PA%.4f,%.4f;PM;PD;\n",	x0, y0 + dy);
- *	  dxf_arc(x0, y0 + dy, x0 + dx, y0 + dy, angle, DELTA);
- *	  fprintf(tfp, "PA%.4f,%.4f;\n", x1 - dx, y0);
- *	  dxf_arc(x1 - dx, y0, x1 - dx, y0 + dy, angle, DELTA);
- *	  fprintf(tfp, "PA%.4f,%.4f;\n", x1, y1 - dy);
- *	  dxf_arc(x1, y1 - dy, x1 - dx, y1 - dy, angle, DELTA);
- *	  fprintf(tfp, "PA%.4f,%.4f;\n", x0 + dx, y1);
- *	  dxf_arc(x0 + dx, y1, x0 + dx, y1 - dy, angle, DELTA);
- *	  fprintf(tfp, "PA%.4f,%.4f;PU;PM2;\n", x0, y0 + dy);
- *
- *	  if (l->thickness != 0) fprintf(tfp, "EP;\n");
- */
-
 	}
 
 	break;
@@ -795,25 +612,25 @@ gendxf_itp_spline(F_spline *s)
 
 	p1 = s->points;
 	cp1 = s->controls;
-	x2 = p1->x/ppi; y2 = p1->y/ppi;
+	x2 = p1->x * figtodxf; y2 = p1->y * figtodxf;
 
 	if (s->thickness != 0 && s->back_arrow)
-	    draw_arrow_head(cp1->rx/ppi, cp1->ry/ppi, x2, y2);
+	    draw_arrow_head(cp1->rx * figtodxf, cp1->ry * figtodxf, x2, y2);
 
 	fprintf(tfp, "PA%.4f,%.4f;PD;\n", x2, y2);
 	for (p2 = p1->next, cp2 = cp1->next; p2 != NULL;
 		p1 = p2, cp1 = cp2, p2 = p2->next, cp2 = cp2->next) {
 	    x1	       = x2;
 	    y1	       = y2;
-	    x2	       = p2->x/ppi;
-	    y2	       = p2->y/ppi;
-	    bezier_spline(x1, y1, (double)cp1->rx/ppi, cp1->ry/ppi,
-		(double)cp2->lx/ppi, cp2->ly/ppi, x2, y2);
+	    x2	       = p2->x * figtodxf;
+	    y2	       = p2->y * figtodxf;
+	    bezier_spline(x1, y1, (double)cp1->rx * figtodxf, cp1->ry *figtodxf,
+		(double)cp2->lx * figtodxf, cp2->ly * figtodxf, x2, y2);
 	    }
 	fprintf(tfp, "PU;\n");
 
 	if (s->thickness != 0 && s->for_arrow)
-	    draw_arrow_head(cp1->lx/ppi, cp1->ly/ppi, x2, y2);
+	    draw_arrow_head(cp1->lx * figtodxf, cp1->ly * figtodxf, x2, y2);
 	}
 
 static void
@@ -849,11 +666,11 @@ gendxf_ctl_spline(F_spline *s)
 	double	x1, y1, x2, y2;
 
 	p = s->points;
-	x1 = p->x/ppi;
-	y1 = p->y/ppi;
+	x1 = p->x * figtodxf;
+	y1 = p->y * figtodxf;
 	p = p->next;
-	x2 = p->x/ppi;
-	y2 = p->y/ppi;
+	x2 = p->x * figtodxf;
+	y2 = p->y * figtodxf;
 	cx1 = (x1 + x2)/2.0;
 	cy1 = (y1 + y2)/2.0;
 	cx2 = (x1 + 3.0*x2)/4.0;
@@ -871,8 +688,8 @@ gendxf_ctl_spline(F_spline *s)
 	for (p = p->next; p != NULL; p = p->next) {
 	    x1 = x2;
 	    y1 = y2;
-	    x2 = p->x/ppi;
-	    y2 = p->y/ppi;
+	    x2 = p->x * figtodxf;
+	    y2 = p->y * figtodxf;
 	    cx3 = (3.0*x1 + x2)/4.0;
 	    cy3 = (3.0*y1 + y2)/4.0;
 	    cx4 = (x1 + x2)/2.0;
@@ -886,8 +703,8 @@ gendxf_ctl_spline(F_spline *s)
 	x1 = x2;
 	y1 = y2;
 	p = s->points->next;
-	x2 = p->x/ppi;
-	y2 = p->y/ppi;
+	x2 = p->x * figtodxf;
+	y2 = p->y * figtodxf;
 	cx3 = (3.0*x1 + x2)/4.0;
 	cy3 = (3.0*y1 + y2)/4.0;
 	cx4 = (x1 + x2)/2.0;
@@ -936,21 +753,21 @@ gendxf_text(F_text *t)
   font	= FONT(t->font);
 
   fprintf(tfp, "  0\nTEXT\n");
-  fprintf(tfp, "  8\n%3i\n",t->depth);
+  fprintf(tfp, "  8\n%3i\n", t->depth);
   fprintf(tfp, "  6\nSolid\n");
-  fprintf(tfp, " 62\n%6i\n",t->color);
-  fprintf(tfp, " 10\n%f\n",t->base_x/ppi);
-  fprintf(tfp, " 11\n%f\n",t->base_x/ppi);
-  fprintf(tfp, " 20\n%f\n",t->base_y/ppi);
-  fprintf(tfp, " 21\n%f\n",t->base_y/ppi);
-  fprintf(tfp, " 50\n%f\n",DPR * t->angle);
-  fprintf(tfp, " 72\n%3i\n",t->type);
+  fprintf(tfp, " 62\n%6i\n", t->color);
+  fprintf(tfp, " 10\n%f\n", t->base_x * figtodxf);
+  fprintf(tfp, " 11\n%f\n", t->base_x * figtodxf);
+  fprintf(tfp, " 20\n%f\n", YCOORD(t->base_y) * figtodxf);
+  fprintf(tfp, " 21\n%f\n", YCOORD(t->base_y) * figtodxf);
+  fprintf(tfp, " 50\n%f\n", DPR * t->angle);
+  fprintf(tfp, " 72\n%3i\n", t->type);
 
   size = t->size;
-  height = size*hcmpp*high[font];
-  fprintf(tfp, " 40\n%f\n",height);
+  height = size*high[font];
+  fprintf(tfp, " 40\n%f\n", height);
 
-  fprintf(tfp, "  1\n%s\n",t->cstring);
+  fprintf(tfp, "  1\n%s\n", t->cstring);
 
 }
 
