@@ -1,7 +1,9 @@
 /*
  * Fig2dev: Translate Fig code to various Devices
- * Parts Copyright (c) 1989-2010 by Brian V. Smith
- * Parts Copyright (c) 2015-2017 by Thomas Loimer
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -14,6 +16,7 @@
  *
  */
 
+
 /*
  * genge.c: convert fig to Graphical Editor format
  *
@@ -29,8 +32,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "fig2dev.h"
-#include "object.h"	/* does #include <X11/xpm.h> */
+#include "fig2dev.h"	/* includes bool.h and object.h */
+//#include "object.h"
+#include "messages.h"
 
 #define		MAX_FONT_SIZE	31
 
@@ -52,7 +56,8 @@ static void	genge_ctl_spline(F_spline *s);
 				    /* color mapping		*/
 				    /* xfig	ge		*/
 
-static int	GE_COLORS[] = {	 1, /* black	black		*/
+static int	GE_COLORS[] = {  1, /* DEFAULT == black		*/
+				 1, /* black	black		*/
 				 8, /* blue	blue		*/
 				 7, /* green	green		*/
 				 6, /* cyan	cyan		*/
@@ -434,7 +439,7 @@ back_arrow(F_line *l)
 static void
 set_color(int col)
 {
-	fprintf(tfp,"c%02d ",GE_COLORS[col]);
+	fprintf(tfp,"c%02d ",GE_COLORS[col + 1]);
 }
 
 /* set fill if there is a fill style */
@@ -443,7 +448,7 @@ static void
 set_fill(int style, int color)
 {
 	if (style != UNFILLED)
-	    fprintf(tfp,"C%02d ",GE_COLORS[color]);
+	    fprintf(tfp,"C%02d ",GE_COLORS[color + 1]);
 }
 
 /*
@@ -484,7 +489,7 @@ struct
 driver dev_ge = {
 	genge_option,
 	genge_start,
-	(void(*)(float,float))gendev_null,
+	gendev_nogrid,
 	genge_arc,
 	genge_ellipse,
 	genge_line,
