@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2018 by Thomas Loimer
+ * Parts Copyright (c) 2015-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -34,9 +34,6 @@
 #include <unistd.h>
 #endif
 #include <math.h>
-#ifdef HAVE_GETPWUID
-#include <pwd.h>
-#endif
 
 #include "fig2dev.h"	/* includes bool.h and object.h */
 //#include "object.h"
@@ -46,7 +43,6 @@
 #include "messages.h"
 #include "pi.h"
 #include "psfonts.h"
-#include "setfigfont.h"
 #include "texfonts.h"
 
 #define UNIT "cm"	/* dip */
@@ -56,22 +52,26 @@ extern char	*ISO1toTeX[];
 extern char	*ISO2toTeX[];
 extern bool	 FontSizeOnly;	/* defined in setfigfont.c */
 
-static void	genpictex_ctl_spline(), genpictex_itp_spline();
-static void	bezier_spline();
-static void	put_box();
-static void	set_style();
-static void	putline();
-static void	rtop();
-static void	draw_arrow_head();
-static void	set_linewidth();
+static void	genpictex_ctl_spline(F_spline *s);
+static void	genpictex_itp_spline(F_spline *s);
+static void	bezier_spline(double a0, double b0, double a1, double b1,
+			double a2, double b2, double a3, double b3);
+static void	put_box(int llx, int lly, int urx, int ury, F_line *l);
+static void	set_style(int style, double dash_len);
+static void	putline(int start_x, int start_y, double end_x, double end_y,
+			int next_x, int next_y,
+			int first_start_x, int first_start_y,
+			int first_end_x, int first_end_y);
+static void	rtop(double x, double y, double *r, double *th);
+static void	draw_arrow_head(double x1, double y1, double x2, double y2,
+			double arrowht, double arrowwid);
+static void	set_linewidth(int w);
 
-static int	encoding = 1;
 static double	dash_length = -1;
 static int	line_style = SOLID_LINE;
 static char	*linethick = "1pt";
 static char	*plotsymbol = "\\makebox(0,0)[l]{\\tencirc\\symbol{'160}}";
 static int	cur_thickness = -1;
-static bool	anonymous = true;
 static bool	rotate = true;
 
 static void
@@ -82,10 +82,6 @@ genpictex_option(char opt, char *optarg)
 	FontSizeOnly = false;
 
 	switch (opt) {
-	case 'a':			/* anonymous (don't output user name) */
-		anonymous = true;
-		break;
-
 	case 'f':			/* set default text font */
 		for (i = 1; i <= MAX_FONT; ++i)
 			if (!strcmp(optarg, texfontnames[i]))
@@ -116,12 +112,6 @@ genpictex_option(char opt, char *optarg)
 		rotate = false;
 		break;
 
-	case 'E':
-		encoding = atoi(optarg);
-		if (encoding < 0 || encoding > 2)
-			encoding = 1;
-		break;
-
 	case 'G':
 	case 'L':
 		break;
@@ -143,9 +133,7 @@ convy(double a)
 void
 genpictex_start(F_compound *objects)
 {
-	char		host[256];
-	struct passwd	*who;
-	char		date_buf[CREATION_TIME_LEN];
+	char	date_buf[CREATION_TIME_LEN];
 
 	texfontsizes[0] = texfontsizes[1] =
 		TEXFONTSIZE(font_size != 0.0? font_size : DEFAULT_FONT_SIZE);
@@ -160,18 +148,6 @@ genpictex_start(F_compound *objects)
 			prog, PACKAGE_VERSION);
 	if (creation_date(date_buf))
 		fprintf(tfp, "%%%%CreationDate: %s\n", date_buf);
-#ifdef HAVE_GETHOSTNAME
-	if (gethostname(host, sizeof(host)) == -1)
-#endif
-		(void)strcpy(host, "unknown-host!?!?");
-#ifdef HAVE_GETPWUID
-	if (!anonymous) {
-		who = getpwuid(getuid());
-		if (who)
-			fprintf(tfp, "%%%%User: %s@%s (%s)\n",
-					who->pw_name, host, who->pw_gecos);
-	}
-#endif
 
 	/* print any whole-figure comments prefixed with "% " */
 	if (objects->comments) {
@@ -180,7 +156,6 @@ genpictex_start(F_compound *objects)
 		fprintf(tfp, "%%\n");
 	}
 	fprintf(tfp, "\\font\\thinlinefont=cmr5\n");
-	define_setfigfont(tfp);
 	fprintf(tfp, "\\mbox{\\beginpicture\n");
 	fprintf(tfp, "\\setcoordinatesystem units <%6.5f%s,%6.5f%s>\n",
 			mag, UNIT, mag, UNIT);
@@ -614,9 +589,8 @@ genpictex_ellipse(F_ellipse *e)
 void
 genpictex_text(F_text *t)
 {
-	double	x, y;
-	char *tpos;
-	unsigned char *cp;
+	double		x, y;
+	char		*tpos;
 
 	/* print any comments */
 	print_comments("% ",t->comments, "");
@@ -646,24 +620,7 @@ genpictex_text(F_text *t)
 		return;
 	}
 
-	unpsfont(t);
-	{
-		int texsize;
-		double baselineskip;
-
-		texsize = TEXFONTMAG(t);
-		baselineskip = (texsize * 1.2);
-
-#ifdef NFSS
-		fprintf(tfp, "\\put{\\SetFigFont{%d}{%.1f}{%s}{%s}{%s}",
-				texsize, baselineskip,
-				TEXFAMILY(t->font), TEXSERIES(t->font),
-				TEXSHAPE(t->font));
-#else
-		fprintf(tfp, "\\put{\\SetFigFont{%d}{%.1f}{%s}",
-				texsize, baselineskip, TEXFONT(t->font));
-#endif
-	}
+	select_font(t, true, true, true);
 
 	if(rotate && t->angle && t->type == T_LEFT_JUSTIFIED)
 		fprintf(tfp,
@@ -672,43 +629,7 @@ genpictex_text(F_text *t)
 
 	set_color(t->color);
 
-	if (!special_text(t))
-		/* this loop escapes characters "$&%#_{}" */
-		/* and deleted characters "~^\" */
-		for(cp = (unsigned char*)t->cstring; *cp; cp++) {
-			if (strchr("$&%#_{}", *cp))
-				(void)fputc('\\', tfp);
-			if (strchr("~^\\", *cp))
-				fprintf(stderr,
-					"Bad character in text object '%c'\n",
-					*cp);
-			else
-				(void)fputc(*cp, tfp);
-		}
-	else
-		for(cp = (unsigned char*)t->cstring; *cp; cp++) {
-#ifdef I18N
-			if (support_i18n && (t->font <= 2))
-				fputc(*cp, tfp);
-			else
-#endif
-				if (*cp >= 0xa0) {
-					switch (encoding) {
-					case 0: /* no escaping */
-						fputc(*cp, tfp);
-						break;
-					case 1: /* iso-8859-1 */
-						fprintf(tfp, "%s",
-							ISO1toTeX[(int)*cp-0xa0]);
-						break;
-					case 2: /* iso-8859-2 */
-						fprintf(tfp, "%s",
-							ISO2toTeX[(int)*cp-0xa0]);
-						break;
-					}
-				} else
-					fputc(*cp, tfp);
-		}
+	put_string(t->cstring, t->font, special_text(t));
 
 	reset_color(t->color);
 
