@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2020 by Thomas Loimer
+ * Parts Copyright (c) 2015-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -50,11 +50,8 @@
 #include <unistd.h>
 #endif
 #include <math.h>
-#include <ctype.h>
+#include <ctype.h>	/* tolower() */
 #include <sys/stat.h>	/* struct stat */
-#ifdef HAVE_GETPWUID
-#include <pwd.h>
-#endif
 #include <locale.h>
 
 #include "fig2dev.h"	/* includes bool.h and object.h */
@@ -67,15 +64,13 @@
 #include "pi.h"
 #include "psfonts.h"
 #include "readpics.h"
+#include "textconvert.h"
 #include "xtmpfile.h"
 
 /* include the PostScript preamble, patterns etc */
 #include "psprolog.h"
 
 extern int	v2_flag, v21_flag, v30_flag;		/* read.c */
-#ifdef I18N
-extern bool	support_i18n;				/* fig2dev.c */
-#endif
 
 /* exported symbols */
 bool		epsflag = false;	/* to distinguish PS and EPS */
@@ -83,11 +78,8 @@ bool		pdfflag = false;	/* to distinguish PDF and PS/EPS */
 
 /*
  * The procedures to embed image files into ps code are defined in a number of
- * source files (readgif.c, readjpg.c, readeps.c,...). These procedures could
- * simply be included here in genps.c, which would result in an overwhelmingly
- * big file. Reading procedures are separated into their respective files, but
- * since they are used only here, the functions are declared here and separate
- * header files are not written.
+ * source files (readgif.c, readjpg.c, readeps.c,...) although these procedures
+ * are only needed in genps.c. The function declarations are given here below.
  */
 #define READ_SIGNATURE \
 	F_pic *pic, struct xfig_stream *restrict pic_stream, int *llx, int *lly
@@ -115,12 +107,9 @@ extern int  read_tif(READ_SIGNATURE);
 extern int  read_xbm(READ_SIGNATURE);
 /* readxpm.c */
 extern int  read_xpm(READ_SIGNATURE);
-#undef READ_SIGNATURE
 
-#ifdef I18N
 static bool	enable_composite_font = false;
-static bool	append_find_composite(FILE *restrict out, FILE *restrict in);
-#endif
+static bool	append_find_composite(FILE *restrict out);
 
 #define		POINT_PER_INCH		72
 #define		ULIMIT_FONT_SIZE	300
@@ -133,7 +122,6 @@ static bool	append_find_composite(FILE *restrict out, FILE *restrict in);
 							obj->thickness > 0)
 
 /* variables obtained from command line options */
-static bool	anonymous = true;
 static bool	asciipreview = false;	/* add ASCII preview? */
 static bool	tiffpreview = false;	/* add a TIFF preview? */
 static bool	tiffcolor = false;	/* color or b/w TIFF preview */
@@ -192,13 +180,12 @@ static void	draw_arrow(F_arrow *arrow, F_pos *points, int npoints,
 				F_pos *fillpoints, int nfillpoints, int col);
 static void	draw_gridline(float x1, float y1, float x2, float y2);
 static bool	ellipse_exist(F_compound *ob);
-static void	encode_all_fonts(F_compound *ob);
 static void	fill_area(int fill, int pen_color, int fill_color);
 static void	genps_ctl_spline(F_spline *s);
 static void	genps_itp_spline(F_spline *s);
 static void	genps_std_colors(void);
 static void	genps_usr_colors(void);
-static bool	iso_text_exist(F_compound *ob);
+static int	note_text_needing_cmap(F_compound *obj);
 static void	putword(int word, FILE *file);
 static void	set_linewidth(double w);
 
@@ -249,6 +236,7 @@ static char	*psfontnames[] = {
 	"Courier"			/* typewriter */
 };
 
+
 #define PS_FONTNAMES(T)	\
 	(((v2_flag&&!(v21_flag||v30_flag)) || \
 		psfont_text(T)) ? PSfontnames : psfontnames)
@@ -274,7 +262,7 @@ static char	*fill_def[NUMPATTERNS] = {
 static	 struct hdr {
 	    char	*type;
 	    char	*bytes;
-	    int		(*readfunc)();
+	    int		(*readfunc)(READ_SIGNATURE);
 	    bool	pipeok;
 	    /* buf[12] below must be large enough for the file signature */
 	} headers[] = {	{"GIF", "GIF",			read_gif,	false},
@@ -439,10 +427,6 @@ gen_ps_eps_option(char opt, char *optarg)
 	case 'P':		/* pagemode */
 		break;
 
-	case 'a':		/* anonymous (don't output user name) */
-	      anonymous = true;
-	      break;
-
 	case 'A':		/* add ASCII preview */
 		asciipreview = true;
 		break;
@@ -504,6 +488,10 @@ gen_ps_eps_option(char opt, char *optarg)
 		}
 		break;
 
+	case 'j':
+		enable_composite_font = true;
+		break;
+
 	case 'l':			/* landscape mode */
 		if (!epsflag) {
 		    landscape = true;	/* override the figure file setting */
@@ -563,6 +551,9 @@ gen_ps_eps_option(char opt, char *optarg)
 		}
 		break;
 
+	case 'Y':			/* option parsed in genpdf_option() */
+		break;
+
 	case 'y':			/* y offset on page */
 		if (!epsflag) {
 		    yoff = atoi(optarg);
@@ -585,19 +576,15 @@ gen_ps_eps_option(char opt, char *optarg)
 void
 genps_start(F_compound *objects)
 {
-	char		 host[256];
 	char		 date_buf[CREATION_TIME_LEN];
-	struct passwd	*who;
 	int		 itmp, jtmp;
 	int		 i;
 	int		 cliplx, cliply, clipux, clipuy;
 	int		 userllx, userlly, userurx, userury;
+	int		 needs_cmap;
 	const struct paperdef	*pd;
 	char		 psize[20];
 
-	char		*libdir;
-	char		 filename[512];
-
 	/* make sure user isn't asking for both TIFF and ASCII preview */
 	if (tiffpreview && asciipreview) {
 		put_msg("Only one type of preview allowed: -A or -T/-C");
@@ -626,10 +613,6 @@ genps_start(F_compound *objects)
 
 	scalex = scaley = mag * POINT_PER_INCH / ppi;
 
-	/* this seems to work around Solaris' cc optimizer bug */
-	/* the problem was that llx had garbage in it - this "fixes" it */
-	sprintf(host,"llx=%d\n",llx);
-
 	/* convert to point unit */
 	fllx = llx * scalex;
 	flly = lly * scaley;
@@ -661,74 +644,74 @@ genps_start(F_compound *objects)
 	}
 
 	if (epsflag) {
-	    /* shift figure to 0,0 */
-	    origx = -fllx;
-	    origy =  fury;
-	    if (boundingboxspec) {
-		    jtmp = sscanf(boundingbox, "%lf %lf %lf %lf",
-				    &userwidthx, &userwidthy,
-				    &userorigx, &userorigy);
-		    switch (jtmp) {
-		    case 0:
-			    userwidthx=(furx-fllx)/POINT_PER_INCH;
-			    if (metric)
-				    userwidthx *= 2.54;
-			    /* the comment below silences gcc's
-			       -Wimplicit-fallthrough warning */
-			    /* intentionally fall through */
-			    /* and set the other user... vars */
-		    case 1:
-			    userwidthy=(fury-flly)/POINT_PER_INCH;
-			    if (metric)
-				    userwidthy *= 2.54;
-			    /* intentionally fall through */
-			    /* and set the other user... vars */
-		    case 2:
-			    userorigx=0;
-			    /* intentionally fall through */
-			    /* and set the last user... var */
-		    case 3:
-			    userorigy=0;
-		    }
-		    if (userwidthx <= 0) {
-			    userwidthx=(furx-fllx)/POINT_PER_INCH;
-			    if (metric)
-				    userwidthx *= 2.54;
-		    }
-		    if (userwidthy <= 0) {
-			    userwidthy=(fury-flly)/POINT_PER_INCH;
-			    if (metric)
-				    userwidthy *= 2.54;
-		    }
+		/* shift figure to 0,0 */
+		origx = -fllx;
+		origy =  fury;
+		if (boundingboxspec) {
+			jtmp = sscanf(boundingbox, "%lf %lf %lf %lf",
+					&userwidthx, &userwidthy,
+					&userorigx, &userorigy);
+			switch (jtmp) {
+			case 0:
+				userwidthx=(furx-fllx)/POINT_PER_INCH;
+				if (metric)
+					userwidthx *= 2.54;
+				/* the comment below silences gcc's
+				   -Wimplicit-fallthrough warning */
+				/* intentionally fall through */
+				/* and set the other user... vars */
+			case 1:
+				userwidthy=(fury-flly)/POINT_PER_INCH;
+				if (metric)
+					userwidthy *= 2.54;
+				/* intentionally fall through */
+				/* and set the other user... vars */
+			case 2:
+				userorigx=0;
+				/* intentionally fall through */
+				/* and set the last user... var */
+			case 3:
+				userorigy=0;
+			}
+			if (userwidthx <= 0) {
+				userwidthx=(furx-fllx)/POINT_PER_INCH;
+				if (metric)
+					userwidthx *= 2.54;
+			}
+			if (userwidthy <= 0) {
+				userwidthy=(fury-flly)/POINT_PER_INCH;
+				if (metric)
+					userwidthy *= 2.54;
+			}
 
-		    userorigx  *= POINT_PER_INCH;
-		    userorigy  *= POINT_PER_INCH;
-		    userwidthx *= POINT_PER_INCH;
-		    userwidthy *= POINT_PER_INCH;
+			userorigx  *= POINT_PER_INCH;
+			userorigy  *= POINT_PER_INCH;
+			userwidthx *= POINT_PER_INCH;
+			userwidthy *= POINT_PER_INCH;
 
-		    if (metric) {
-			    userorigx  /= 2.54;
-			    userorigy  /= 2.54;
-			    userwidthx /= 2.54;
-			    userwidthy /= 2.54;
-		    }
+			if (metric) {
+				userorigx  /= 2.54;
+				userorigy  /= 2.54;
+				userwidthx /= 2.54;
+				userwidthy /= 2.54;
+			}
 
-		    userllx = (int) floor(userorigx);
-		    userlly = (int) floor(userorigy);
-		    userurx = (int) ceil(userorigx+userwidthx);
-		    userury = (int) ceil(userorigy+userwidthy);
+			userllx = (int) floor(userorigx);
+			userlly = (int) floor(userorigy);
+			userurx = (int) ceil(userorigx+userwidthx);
+			userury = (int) ceil(userorigy+userwidthy);
 
-		    /* adjust for any border margin */
-		    userllx -= border_margin;
-		    userlly -= border_margin;
-		    userurx += border_margin;
-		    userury += border_margin;
+			/* adjust for any border margin */
+			userllx -= border_margin;
+			userlly -= border_margin;
+			userurx += border_margin;
+			userury += border_margin;
 
-		    if (useabsolutecoo) {
-			    userllx += origx;
-			    userurx += origx;
-		    }
-	    }
+			if (useabsolutecoo) {
+				userllx += origx;
+				userurx += origx;
+			}
+		}
 	} else {	/* postscript */
 		if (landscape) {
 			itmp = pageheight;
@@ -755,24 +738,12 @@ genps_start(F_compound *objects)
 	else
 		fputs("%!PS-Adobe-3.0\n", tfp);
 
-#ifdef HAVE_GETHOSTNAME
-	if (gethostname(host, sizeof(host)) == -1)
-#endif
-		(void) strcpy(host, "unknown host");
 	fprintf(tfp, "%%%%Title: %s\n",
 		(name? name: ((from) ? from : "stdin")));
 	fprintf(tfp, "%%%%Creator: %s Version %s\n",
 		prog, PACKAGE_VERSION);
 	if (creation_date(date_buf))
 		fprintf(tfp, "%%%%CreationDate: %s\n", date_buf);
-#ifdef HAVE_GETPWUID
-	if (!anonymous) {
-		who = getpwuid(getuid());
-		if (who)
-			fprintf(tfp, "%%%%For: %s@%s (%s)\n",
-				who->pw_name, host, who->pw_gecos);
-	}
-#endif
 
 	/* calc initial clipping area to size of the bounding box
 	   (this is needed for later clipping by arrowheads) */
@@ -849,6 +820,12 @@ genps_start(F_compound *objects)
 
 	/* put in the magnification for information purposes */
 	fprintf(tfp, "%%%%Magnification: %.4f\n",metric? mag*76.2/80.0 : mag);
+
+	if ((needs_cmap = note_text_needing_cmap(objects))) {
+		fputs("%%DocumentNeededResources: ProcSet (CIDInit)\n", tfp);
+		fputs("%%+ CMap (Xfig-UTF8-H)\n", tfp);
+		fputs("%%DocumentSuppliedResources: CMap (Xfig-UTF8-H)\n", tfp);
+	}
 	fputs("%%EndComments\n", tfp);
 
 	/* This %%BeginSetup .. %%EndSetup has to occur after
@@ -893,21 +870,14 @@ genps_start(F_compound *objects)
 		fputs("%\n", tfp);
 	}
 
-	/* insert PostScript codes to select paper size, if exist */
-	libdir = getenv("FIG2DEV_LIBDIR");
-#ifdef I18N_DATADIR
-	if (libdir == NULL)
-		libdir = I18N_DATADIR;
-#endif
-	if (libdir != NULL) {
-		sprintf(filename, "%s/%s.ps", libdir, papersize);
-		/* get filename like "/usr/local/lib/fig2dev/A3.ps" and
-		   prepend it to the postscript code;
-		   do not mind, if it does not work */
-		(void)append(filename, tfp);
+	fputs("%%BeginProlog\n", tfp);
+
+	if (needs_cmap) {
+		fputs(XFIG_CMAP1, tfp);
+		fputs(XFIG_CMAP2, tfp);
+		fputs(XFIG_CMAP3, tfp);
 	}
 
-	fputs("%%BeginProlog\n", tfp);
 	if (pats_used)
 		fprintf(tfp,"/MyAppDict 100 dict dup begin def\n");
 	fprintf(tfp, "%s", BEGIN_PROLOG1);
@@ -939,72 +909,36 @@ genps_start(F_compound *objects)
 
 	/* translate (in multi-page mode this is done at end of this proc) */
 	/* (rotation and y flipping is done in %%BeginPageSetup area */
-	if (pats_used) {
-		int i;
+	if (pats_used)
 		/* only define the patterns that are used */
 		for (i=0; i<NUMPATTERNS; i++)
 			if (pattern_used[i])
 				fprintf(tfp, "\n%s", fill_def[i]);
-	}
 	fprintf(tfp, "\n%s", BEGIN_PROLOG2);
-	if (iso_text_exist(objects)) {
-		fprintf(tfp, "%s%s%s",
-			SPECIAL_CHAR_1, SPECIAL_CHAR_2, SPECIAL_CHAR_3);
-		encode_all_fonts(objects);
-	}
+
+	if (needs_cmap)
+		/*
+		 * for each font insert, e.g.,
+		 *	/NewCenturySchlbk-Roman-UTF-8
+		 *	/Xfig-AGL-UTF8-H /CMap findresource
+		 *	[/NewCenturySchlbk-Roman ] composefont pop
+		 */
+		for (i = 0; i < MAX_PSFONT + 2; ++i)
+			if (PSneedsutf8[i] == 1)
+				fprintf(tfp, "/%s-UTF-8\n/Xfig-AGL-UTF8-H /CMap"
+						" findresource\n"
+						"[/%s ] composefont pop\n",
+						PSfontnames[i],
+						PSfontnames[i]);
+
 	if (ellipse_exist(objects))
 		fprintf(tfp, "%s\n", ELLIPSE_PS);
 	if (approx_spline_exist(objects))
 		fprintf(tfp, "%s\n", SPLINE_PS);
-#ifdef I18N
-	if (support_i18n && iso_text_exist(objects)) {
-		char *libdir, *locale;
-		char localefile_buf[128];
-		char *localefile = localefile_buf;
-		FILE *fp;
-		libdir = getenv("FIG2DEV_LIBDIR");
-#ifdef I18N_DATADIR
-		if (libdir == NULL)
-			libdir = I18N_DATADIR;
-#endif
-		locale = setlocale(LC_CTYPE, NULL);
-		if (locale == NULL) {
-			fprintf(stderr,
-			      "fig2dev: LANG not defined; assuming C locale\n");
-			locale = "C";
-		}
-		if (strlen(libdir) + strlen(locale) + 5 > sizeof localefile_buf)
-			localefile = malloc(strlen(libdir) + strlen(locale) + 5);
-		if (localefile != NULL) {
-			sprintf(localefile, "%s/%s.ps", libdir, locale);
-			/* get filename like
-			   ``/usr/local/lib/fig2dev/japanese.ps'' */
-			fp = fopen(localefile, "rb");
-			if (fp == NULL) {
-				fprintf(stderr, "fig2dev: can not open file: %s\n",
-						localefile);
-			} else {
-				enable_composite_font =
-					append_find_composite(tfp, fp);
 
-				if (ferror(tfp)) {
-					fputs("Error writing output file.\n",
-							stderr);
-					exit(EXIT_FAILURE);
-				}
-				if (ferror(fp)) {
-					fprintf(stderr,
-						"Error reading file %s.\n"
-						"The output might be broken.\n",
-						localefile);
-				}
-			fclose(fp);
-			}
-		}
-		if (localefile != localefile_buf)
-			free(localefile);
-	}
-#endif /* I18N */
+	if (needs_cmap && enable_composite_font)
+		/* append the composite font prologue, if applicable */
+		enable_composite_font = append_find_composite(tfp);
 
 	fprintf(tfp, "%s\n", END_PROLOG);
 
@@ -1039,7 +973,6 @@ genps_start(F_compound *objects)
 	fputs("restore\n", tfp);
 	fputs("} bind def\n", tfp);
 
-
 	if (multi_page) {
 		/* reset the matrix for multipage mode */
 		fputs("initmatrix\n", tfp);
@@ -1181,6 +1114,7 @@ genps_end(void)
 	const int	h = pageheight, w = pagewidth;
 	int		epslen, tiflen;
 	struct stat	fstat;
+	char		date_buf[CREATION_TIME_LEN];
 
 	/* for multipage, translate and output objects for each page */
 	if (multi_page) {
@@ -1368,6 +1302,13 @@ genps_end(void)
 	/* final DSC comment for eps output (EOF = end of document) */
 	fputs("%EOF\n", tfp);
 
+	if (pdfflag) {
+		if (creation_date_pdfmark(date_buf))
+		        fprintf(tfp,
+				"[ /ModDate (%s)\n /CreationDate (%s)\n /DOCINFO pdfmark\n",
+				date_buf, date_buf);
+	}
+
 	/* all ok */
 	return 0;
 }
@@ -1411,15 +1352,53 @@ append(const char *restrict infilename, FILE *outfile)
 	return 0;
 }
 
-#ifdef I18N
+#define COMPOSITE_ERRMSG	"fig2dev: LC_CTYPE not defined.\n\
+  In order to use composed fonts for japanese or korean output,\n\
+  set LC_CTYPE to one of the locales ja_JP.eucJP, ja_JP.EUC, ja_JP,\n\
+  ja_JP.ujis, ja, japanese, ko_KR.eucKR, ko_KR.EUC, ko_KR, ko, or korean.\n"
+
 /*
- * Append open file in to open file out while searching for the string
- * "CompositeRoman".  Return true if found, false otherwise.
+ * Search for a postscript prologue based on the current locale.
+ * Append the file to out.
+ * Search for the text "CompositeRoman" and return true, if found.
+ * Return false if not found or if the postscript prologue file was not found.
  */
 static bool
-append_find_composite(FILE *restrict out, FILE *restrict in)
+append_find_composite(FILE *restrict out)
 {
+	char	*libdir, *locale;
+	char	localefile_buf[128];
+	char	*localefile = localefile_buf;
+	FILE	*in;
+
+	libdir = getenv("FIG2DEV_LIBDIR");
+#ifdef I18N_DATADIR
+	if (libdir == NULL)
+		libdir = I18N_DATADIR;
+#endif
+	locale = setlocale(LC_CTYPE, NULL);
+	if (locale == NULL) {
+		fputs(COMPOSITE_ERRMSG, stderr);
+		return false;
+	}
+	if (strlen(libdir) + strlen(locale) + 5 > sizeof localefile_buf &&
+		!(localefile = malloc(strlen(libdir) + strlen(locale) + 5))) {
+		put_msg(Err_mem);
+		return false;
+	}
+
+	sprintf(localefile, "%s/%s.ps", libdir, locale);
+	/* get filename like /usr/local/lib/fig2dev/japanese.ps */
+	if (!(in = fopen(localefile, "rb"))) {
+		err_msg("Unable to open postscript prologue file %s");
+		if (localefile != localefile_buf)
+			free(localefile);
+		return false;
+	}
+
 	/*
+	 * Now search for CompositeRoman and, if found, copy the file to tfp.
+	 *
 	 * Initially, the needle was passed as a function parameter. However,
 	 * Visual Studio cannot create an array of a size depending on a
 	 * function parameter (str[], below). Therefore, use the fixed needle
@@ -1458,10 +1437,19 @@ append_find_composite(FILE *restrict out, FILE *restrict in)
 		}
 		fwrite(str + needle_len, (size_t)1, chars, out);
 	}
+	if (ferror(out)) {
+		put_msg("Error writing output file.");
+		exit(EXIT_FAILURE);
+	}
+	if (!fclose(in)) {
+		err_msg("Error reading file %s", localefile);
+		put_msg("The output might be broken.");
+	}
+
+	if (localefile != localefile_buf)
+		free(localefile);
 	return found;
 }
-#endif /* I18N */
-
 
 /* read file named in "infilename", converting the binary to hex and
    append to already open FILE "outfile".
@@ -1492,7 +1480,6 @@ appendhex(char *infilename, FILE *outfile, int width, int height)
 	fclose(infile);
 }
 
-
 static void
 set_style(int s, double v)
 {
@@ -1571,7 +1558,7 @@ genps_line(F_line *l)
 	do_split(l->depth);
 
 	if (multi_page)
-	   fprintf(tfp, "/o%d {", no_obj++);
+		fprintf(tfp, "/o%d {", no_obj++);
 
 	/* print any comments prefixed with "%" */
 	print_comments("% ",l->comments, "");
@@ -1661,9 +1648,9 @@ genps_line(F_line *l)
 		fputs("% pen to black in case this eps object doesn't set "
 			"color first\n", tfp);
 		if (grayonly)
-		    fputs("0 setgray\n", tfp);
+			fputs("0 setgray\n", tfp);
 		else
-		    fputs("0 0 0 setrgbcolor\n", tfp);
+			fputs("0 0 0 setrgbcolor\n", tfp);
 
 		init_stream(&pic_stream);
 
@@ -1942,17 +1929,12 @@ genps_line(F_line *l)
 		/* second point */
 		fpntx2 = q->x;
 		fpnty2 = q->y;
-		/* go through the points to get the last two */
-		while (q->next != NULL) {
-			p = q;
-			q = q->next;
-		}
 		/* next to last point */
-		lpntx2 = p->x;
-		lpnty2 = p->y;
+		lpntx2 = l->last[1].x;
+		lpnty2 = l->last[1].y;
 		/* last point */
-		lpntx1 = q->x;
-		lpnty1 = q->y;
+		lpntx1 = l->last[0].x;
+		lpnty1 = l->last[0].y;
 		/* set clipping for any arrowheads */
 		if (NEEDS_CLIPPING(l)) {
 			fprintf(tfp, "gs ");
@@ -1960,8 +1942,6 @@ genps_line(F_line *l)
 		}
 
 		/* now output the points */
-		p = l->points;
-		q = p->next;
 		fprintf(tfp, "n %d %d m", p->x, p->y);
 		i=1;
 		while (q->next != NULL) {
@@ -2358,15 +2338,15 @@ genps_ellipse(F_ellipse *e)
 void
 genps_text(F_text *t)
 {
-	unsigned char		*cp;
-#ifdef I18N
+	unsigned char	*cp;
 #define LINE_LENGTH_LIMIT 200
-	bool composite = false;
-	bool state_gr = false;
-	int chars = 0;
-	int gr_chars = 0;
-	unsigned char ch;
-#endif /* I18N */
+	bool		composite = false;
+	bool		state_gr = false;
+	int		chars = 0;
+	int		gr_chars = 0;
+	unsigned char	ch;
+	char		*non_ascii = NULL;
+	char		*str = NULL;
 
 	do_split(t->depth);
 
@@ -2380,7 +2360,6 @@ genps_text(F_text *t)
 	/* print any comments prefixed with "%" */
 	print_comments("% ",t->comments, "");
 
-#ifdef I18N
 	if (enable_composite_font && ((t->flags & PSFONT_TEXT) ?
 				(t->font <= 0 || t->font == 2) :
 				(t->font <= 2))) {
@@ -2390,22 +2369,37 @@ genps_text(F_text *t)
 					PSFONTMAG(t));
 		else
 			fprintf(tfp, TEXT_PS, "CompositeBold", "",PSFONTMAG(t));
-	} else
-#endif /* I18N */
-		if (PSisomap[t->font+1] == true)
-			fprintf(tfp, TEXT_PS, PSFONT(t), "-iso", PSFONTMAG(t));
+	} else {
+		if (PSneedsutf8[t->font + 1] == 1 &&
+				(non_ascii = conv_non_ascii(t->cstring)))
+			fprintf(tfp, TEXT_PS, PSFONT(t), "-UTF-8",PSFONTMAG(t));
 		else
 			fprintf(tfp, TEXT_PS, PSFONT(t), "", PSFONTMAG(t));
+	}
 
 	fprintf(tfp, "%d %d m\ngs ", t->base_x,  t->base_y);
 	fprintf(tfp, "1 -1 sc ");
 
 	if (t->angle != 0.0)
 		fprintf(tfp, " %.1f rot ", t->angle*180.0/M_PI);
-	/* this loop escapes characters '(', ')', and '\' */
+
 	fputc('(', tfp);
-#ifdef I18N
-	for(cp = (unsigned char *)t->cstring; *cp; cp++) {
+
+	if ((!enable_composite_font || !composite) && non_ascii
+			/* && PSneedsutf8[] == 1 */) {
+		(void)conv_textstring(&str, t->cstring, strlen(t->cstring));
+	} else {
+		char	*c;
+		/* PSneedsutf8[] == 2 and composite are mutually exclusive */
+		str = t->cstring;
+		if (PSneedsutf8[t->font + 1] == 2 && (c = conv_textisutf8(str)))
+			/* if need_conversion == 1, the string is not
+			   utf8-encoded */
+			(void)convertutf8tolatin1(c);
+	}
+
+	/* this loop escapes characters '(', ')', and '\' */
+	for(cp = (unsigned char *)str; *cp; cp++) {
 		if (LINE_LENGTH_LIMIT < chars) {
 			fputs("\\\n", tfp);
 			chars = 0;
@@ -2414,6 +2408,8 @@ genps_text(F_text *t)
 		if (enable_composite_font && composite) {
 			if (ch & 0x80) {	/* GR */
 				if (!state_gr) {
+					/* insert the EscChar (default 255) and
+					   the font number (here 1) */
 					fprintf(tfp, "\\377\\001");
 					chars += 8;
 					state_gr = true;
@@ -2428,6 +2424,7 @@ genps_text(F_text *t)
 								t->cstring);
 						fputc('?', tfp);
 					}
+					/* EscChar and font number, here 0 */
 					fprintf(tfp, "\\377\\000");
 					chars += 8;
 					state_gr = false;
@@ -2438,7 +2435,7 @@ genps_text(F_text *t)
 			fputc('\\', tfp);
 			chars += 1;
 		}
-		if (ch>=0x80) {
+		if (0/*ch>=0x80*/) {
 			fprintf(tfp,"\\%o", ch);
 			chars += 4;
 		} else {
@@ -2453,17 +2450,9 @@ genps_text(F_text *t)
 			fputc('?', tfp);
 		}
 	}
-#else
-	for(cp = (unsigned char *)t->cstring; *cp; cp++) {
-		if (strchr("()\\", *cp))
-			fputc('\\', tfp);
-		if (*cp>=0x80)
-			fprintf(tfp,"\\%o", *cp);
-		else
-			fputc(*cp, tfp);
-	}
-#endif /* I18N */
 	fputc(')', tfp);
+	if (str && str != t->cstring)
+		free(str);
 
 	if ((t->type == T_CENTER_JUSTIFIED) || (t->type == T_RIGHT_JUSTIFIED)){
 
@@ -2489,6 +2478,9 @@ draw_arrow(F_arrow *arrow, F_pos *points, int npoints,
 {
 	int i, type;
 
+	if (npoints < 2)
+		return;
+
 	fprintf(tfp,"%% arrowhead\n");
 	set_linecap(0);			/* butt line cap for arrowheads */
 	set_linejoin(0);		/* miter join for sharp points */
@@ -2739,52 +2731,29 @@ genps_usr_colors(void)
 	}
 }
 
-static bool
-iso_text_exist(F_compound *ob)
+static int
+note_text_needing_cmap(F_compound *obj)
 {
-	F_compound	*c;
-	F_text	    *t;
-	unsigned char   *s;
+	int		needs_cmap = 0;
+	F_compound	*o;
+	F_text		*t;
 
-	if (ob->texts != NULL) {
-		for (t = ob->texts; t != NULL; t = t->next) {
-			/* look for any ISO (non-ASCII) chars in
-			   non-special text except for pstex */
-			if (!strcmp(lang,"pstex") && special_text(t))
-				continue;
-			for (s = (unsigned char*)t->cstring; *s != '\0'; s++) {
-				/* look for characters >= 128 or ASCII '-' */
-				if ((*s>127) || (*s=='-'))
-					return true;
-			}
+	for (t = obj->texts; t != NULL; t = t->next) {
+		if (PSneedsutf8[t->font + 1] ||
+			((!strcmp(lang, "pstex") || !strcmp(lang, "pdftex")) &&
+				 special_text(t)))
+			continue;
+		if (!PSneedsutf8[t->font + 1] && conv_non_ascii(t->cstring)) {
+			if (!needs_cmap)
+				needs_cmap = 1;
+			PSneedsutf8[t->font + 1] = 1;
 		}
 	}
 
-	for (c = ob->compounds; c != NULL; c = c->next) {
-		if (iso_text_exist(c))
-			return true;
-	}
-	return false;
-}
+	for (o = obj->compounds; o != NULL; o = o->next)
+		needs_cmap = note_text_needing_cmap(o) || needs_cmap;
 
-static void
-encode_all_fonts(F_compound *ob)
-{
-	F_compound *c;
-	F_text     *t;
-
-	if (ob->texts != NULL) {
-		for (t = ob->texts; t != NULL; t = t->next)
-			if (PSisomap[t->font+1] == false) {
-				fprintf(tfp, "/%s /%s-iso isovec ReEncode\n",
-						PSFONT(t), PSFONT(t));
-				PSisomap[t->font+1] = true;
-			}
-	}
-
-	for (c = ob->compounds; c != NULL; c = c->next) {
-		encode_all_fonts(c);
-	}
+	return needs_cmap;
 }
 
 static bool
