@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2019 by Thomas Loimer
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -19,9 +19,11 @@
 #ifndef GENPS_H
 #define GENPS_H
 
+#include "bool.h"
+#include "object.h"
+
 extern bool	epsflag;	/* to distinguish PS and EPS */
 extern bool	pdfflag;	/* to distinguish PDF and PS/EPS */
-extern bool	tiffpreview;	/* add a TIFF preview? */
 
 extern void	gen_ps_eps_option(char opt, char *optarg);
 extern void	genps_start(F_compound *objects);
