@@ -3,7 +3,7 @@
  * Parts Copyright (c) 2002 by Anthony Starks
  * Parts Copyright (c) 2002-2006 by Martin Kroeker
  * Parts Copyright (c) 2002-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2020 by Thomas Loimer
+ * Parts Copyright (c) 2015-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -636,7 +636,7 @@ gensvg_line(F_line *l)
     if (l->type == T_PIC_BOX ) {
 	fprintf(tfp,"<!-- Image -->\n");
 	fprintf(tfp,
-	    "<image xlink:href=\"file://%s\" preserveAspectRatio=\"none\"\n",
+	    "<image xlink:href=\"file:%s\" preserveAspectRatio=\"none\"\n",
 	    l->pic->file);
 	p = l->points;
 	px = p->x;
@@ -986,7 +986,7 @@ gensvg_text(F_text *t)
 		"font-style=\"%s\" font-weight=\"%s\" font-size=\"%d\" text-anchor=\"%s\">",
 		((t->font % 2 == 0 || t->font > 31) ? "normal" : "italic"),
 		((t->font % 4 < 2 || t->font > 31) ? "normal" : "bold"),
-		(int)ceil(t->size * 12), anchor[t->type]);
+		(int)(t->size * ppi / 72. + 0.5), anchor[t->type]);
 
 	if (t->font == 32) {
 		for (cp = (unsigned char *)t->cstring; *cp; ++cp) {
@@ -1005,7 +1005,7 @@ gensvg_text(F_text *t)
 #endif
 		for (cp = (unsigned char *)t->cstring; *cp; cp++) {
 			ch = *cp;
-			if (( supsub == 2 &&ch == '}' ) || supsub==1) {
+			if ((supsub == 2 && ch == '}') || supsub==1) {
 #ifdef NOSUPER
 				fprintf(tfp,"</tspan><tspan dy=\"%d\">",-dy);
 				old_dy=-dy;
@@ -1019,6 +1019,8 @@ gensvg_text(F_text *t)
 				}
 			}
 			if (ch == '_' || ch == '^') {
+				if (*(cp + 1) == '\0')
+					break;
 				supsub=1;
 #ifdef NOSUPER
 				if (dy != 0)
@@ -1043,6 +1045,8 @@ gensvg_text(F_text *t)
 				++cp;
 				ch = *cp;
 				if (ch == '{' ) {
+					if (*(cp + 1) == '\0')
+						break;
 					supsub=2;
 					++cp;
 					ch = *cp;
@@ -1169,7 +1173,7 @@ svg_arrows(int line_thickness, F_arrow *for_arrow, F_arrow *back_arrow,
 	return true;
     }
 
-    if (for_arrow) {
+    if (for_arrow && fnpoints > 1) {
 	fputs("<!-- Forward arrow", tfp);
 	arrow_path(for_arrow, forw2, pen_color, fnpoints, fpoints,
 		fnfillpoints, ffillpoints
@@ -1178,7 +1182,7 @@ svg_arrows(int line_thickness, F_arrow *for_arrow, F_arrow *back_arrow,
 #endif
 		);
     }
-    if (back_arrow) {
+    if (back_arrow && bnpoints > 1) {
 	fputs("<!-- Backward arrow", tfp);
 	arrow_path(back_arrow, back2, pen_color, bnpoints, bpoints,
 		bnfillpoints, bfillpoints
