@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2020 by Thomas Loimer
+ * Parts Copyright (c) 2015-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -52,7 +52,8 @@ int ReadFromBitmapFile(FILE *file, unsigned int *width, unsigned int *height,
 		unsigned char **data_ret);	/* readxbm.c */
 
 static void
-	drawFilledShape(void (*)(), void *, int, int, int, int, int, double),
+	drawFilledShape(void (*)(void *), void *, int, int, int, int, int,
+			double),
 	drawBitmap(F_line *),
 	tkLine(F_line *, int, int, int, double),
 	tkArc(void *),
@@ -274,7 +275,7 @@ gentk_start(F_compound *objects)
  */
 
 int
-gentk_end()
+gentk_end(void)
 {
 	fprintf(tfp, "focus %s\n", canvas);
 
@@ -484,8 +485,8 @@ drawBitmap(F_line *l)
 void
 gentk_text(F_text * t)
 {
-	char		stfp[2048];
-	unsigned int	i, j;
+	extern FILE	*tfp;
+	size_t		i;
 
 	/* I'm sure I'm just too dense to have seen a better way of doing this... */
 	static struct {
@@ -572,42 +573,36 @@ gentk_text(F_text * t)
 	if (t->angle != 0.)
 	    fprintf(stderr, "gentk_text: rotated text not supported by Tk.\n");
 
-	sprintf(stfp, "%s create text %fi %fi", canvas, X(t->base_x),
+	fprintf(tfp, "%s create text %fi %fi", canvas, X(t->base_x),
 		Y(t->base_y));
-	niceLine(stfp);
-	strcpy(stfp, " -text \"");
-	j = strlen(stfp);
+	fputs(" -text \"", tfp);
 	for (i = 0; i < strlen(t->cstring); ++i) {
 		if (t->cstring[i] == '"')
-			stfp[j++] = '\\';
-		stfp[j++] = t->cstring[i];
+			fputc('\\', tfp);
+		fputc(t->cstring[i], tfp);
 	}
-	stfp[j++] = '"';
-	stfp[j++] = '\0';
-	niceLine(stfp);
+	fputs("\" \\\n", tfp);
 	switch (t->type) {
 	case T_LEFT_JUSTIFIED:
 	case DEFAULT:
-		sprintf(stfp, " -anchor sw");
+		fprintf(tfp, " -anchor sw");
 		break;
 	case T_CENTER_JUSTIFIED:
 		/* The Tk default. */
-		sprintf(stfp, " -anchor s");
+		fprintf(tfp, " -anchor s");
 		break;
 	case T_RIGHT_JUSTIFIED:
-		sprintf(stfp, " -anchor se");
+		fprintf(tfp, " -anchor se");
 		break;
 	default:
 		fprintf(stderr, "gentk_text: Unknown text justification\n");
 		t->type = T_LEFT_JUSTIFIED;
 		break;
 	}
-	niceLine(stfp);
 
 	if (psfont_text(t)) {
-		sprintf(stfp, " -font \"%s%d%s\"", fontNames[t->font+1].prefix,
+		fprintf(tfp, " -font \"%s%d%s\"", fontNames[t->font+1].prefix,
 			(int) (t->size*mag), fontNames[t->font+1].suffix);
-		niceLine(stfp);
 	} else {	/* Rigid, special, and LaTeX fonts. */
 		int fnum;
 
@@ -635,13 +630,11 @@ gentk_text(F_text * t)
 			fprintf(stderr, "gentk_text: unknown LaTeX font.\n");
 			break;
 		}
-		sprintf(stfp, " -font \"%s%d%s\"", fontNames[fnum].prefix,
+		fprintf(tfp, " -font \"%s%d%s\"", fontNames[fnum].prefix,
 			(int) (t->size*mag), fontNames[fnum].suffix);
-		niceLine(stfp);
 	}
 	if (t->color != BLACK_COLOR && t->color != DEFAULT) {
-		sprintf(stfp, " -fill #%6.6x", rgbColorVal(t->color));
-		niceLine(stfp);
+		fprintf(tfp, " -fill #%6.6x", rgbColorVal(t->color));
 	}
 
 	fputc('\n', tfp);
@@ -993,7 +986,7 @@ void gentk_spline(F_spline *s)
  */
 
 static void
-drawFilledShape(void (*tkShape)(), void *p, int thickness, int penColor,
+drawFilledShape(void (*tkShape)(void *), void *p, int thickness, int penColor,
 	int fillColor, int fillStyle, int style, double style_val)
 {
 	char	sftp[256];
@@ -1234,33 +1227,33 @@ tkLine(F_line *l, int penColor, int thickness, int style, double style_val)
 	tk_setstyle(style, style_val);
 
 	switch (l->join_style) {
-	    case 0:	/* Miter (Tk default). */
+	case 0:	/* Miter (Tk default). */
 		break;
-	    case 1:	/* Round. */
+	case 1:	/* Round. */
 		sprintf(stfp, " -joinstyle round");
 		niceLine(stfp);
 		break;
-	    case 2:	/* Bevel. */
+	case 2:	/* Bevel. */
 		sprintf(stfp, " -joinstyle bevel");
 		niceLine(stfp);
 		break;
-	    default:
+	default:
 		fputs("tkLine: unknown join style.\n", stderr);
 		break;
 	}
 
 	switch (l->cap_style) {
-	    case 0:	/* Butt (Tk default). */
+	case 0:	/* Butt (Tk default). */
 		break;
-	    case 1:	/* Round. */
+	case 1:	/* Round. */
 		sprintf(stfp, " -capstyle round");
 		niceLine(stfp);
 		break;
-	    case 2: /* Projecting. */
+	case 2: /* Projecting. */
 		sprintf(stfp, " -capstyle projecting");
 		niceLine(stfp);
 		break;
-	    default:
+	default:
 		fputs("tkLine: unknown cap style.\n", stderr);
 		break;
 	}
