@@ -2,8 +2,8 @@
  * Fig2dev: Translate Fig code to various Devices
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2002 by Brian V. Smith
- * Parts Copyright (c) 2015 by Thomas Loimer
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2019 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,6 +16,9 @@
  *
  */
 
+#ifndef PICFONTS_H
+#define PICFONTS_H
+
 /*  The selection of font names may be site dependent */
 
 char		*picfontnames[] = {
@@ -26,10 +29,14 @@ char		*picfontnames[] = {
 			"H",			/* sans serif */
 			"C"			/* typewriter */
 		};
+
 #define MAXFONTSIZE 108
 
 #define PICFONT(F)	(picfontnames[((F) <= MAX_FONT) ? (F)+1 : MAX_FONT])
 #define PICFONTSIZE(S)  ((S) > 0 ? \
 				((S) <= MAXFONTSIZE ? (int)round(S) : \
 					MAXFONTSIZE) : (int)font_size)
-#define PICFONTMAG(T)	PICFONTSIZE((int)(T->size*(rigid_text(T) ? 1.0 : fontmag)))
+#define PICFONTMAG(T)	\
+		PICFONTSIZE((int)(T->size*(rigid_text(T) ? 1.0 : fontmag)))
+
+#endif /* PICFONTS_H */
