@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2021 by Thomas Loimer
+ * Parts Copyright (c) 2015-2024 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -25,6 +25,8 @@
 #include "config.h"
 #endif
 
+#include <errno.h>
+#include <math.h>			/* isfinite() */
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -79,13 +81,11 @@ bool	multispec = false;	/* set if the user specs. multiple pages */
 bool	metric;			/* true if file specifies Metric */
 bool	grayonly = false;	/* convert colors to grayscale (-N option) */
 bool	bgspec = false;		/* flag to say -g was specified */
-#ifdef I18N
-bool support_i18n = false;
-#endif
 char	gif_transparent[]="\0"; /* GIF transp color hex name (e.g. #ff00dd) */
 char	papersize[PAPERSZ_LEN];	/* paper size */
 char	boundingbox[64];	/* boundingbox */
 char	lang[12];		/* selected output language */
+char	*input_encoding = NULL;
 RGB	background;		/* background (if specified by -g) */
 float	grid_minor_spacing = 0.0; /* grid minor spacing (if any) */
 float	grid_major_spacing = 0.0; /* grid major spacing (if any) */
@@ -143,12 +143,13 @@ bool	arrows_used, arrow_used[NUMARROWS];
 
 /* local */
 struct obj_rec {
-	void (*gendev)();
-	char *obj;
+	void (*gendev)(void *obj);
+	void *obj;
 	int depth;
 };
-static bool	maxdimspec = false; /* if max size of figure (-Z) was given */
-static float	max_dimension;	/* max. dimension (-Z) of figure */
+static int	maxdimspec = 0; /* if max size of figure (-Z) was given */
+			/* 1 - max dimension, 2 - width, 3 - height specified */
+static double	max_dimension;	/* specified dimension (-Z) of figure */
 static float	mult;		/* multiplier for grid spacing */
 static struct driver	*dev = NULL;
 static int	depth_index = 0;
@@ -160,11 +161,9 @@ static struct depth_opts {
 } depth_opt[NUMDEPTHS + 1];
 
 static char	Usage[] =
-#ifdef I18N
-"Usage:\n %1$s -hV\n %1$s -L language [-s size] [-m scale] [-j] [input [output]]\n";
-#else
-"Usage:\n %1$s -hV\n %1$s -L language [-s size] [-m scale] [input [output]]\n";
-#endif
+"Usage:\n %1$s -hV\n"
+" %1$s -L language -h\n"
+" %1$s [-L language] [other options] [in.fig [output]]\n";
 
 static int	 parse_gridspec(char *string, float *numer, float *denom,
 				float *spacing, int *nchrs);
@@ -187,14 +186,6 @@ print_comments(char *string1, F_comment *comment, char *string2)
 	}
 }
 
-/* all option letters must be in this string */
-/* not in this string: HIJQUu and non-alphabetic chars */
-#ifdef I18N
-#define ARGSTRING	"AaB:b:C:cD:d:E:eFf:G:g:hi:jKkL:l:Mm:Nn:OoPp:q:R:rS:s:Tt:VvWwX:x:Y:y:Z:z:?"
-#else
-#define ARGSTRING	"AaB:b:C:cD:d:E:eFf:G:g:hi:KkL:l:Mm:Nn:OoPp:q:R:rS:s:Tt:VvWwX:x:Y:y:Z:z:?"
-#endif
-
 /* Options with `continue;' are not passed to drivers;
  * Therefore, only -L and -G remain to be cared for in drivers.
  * -L is needed in genepic and genmp while in genbitmaps, lang is used.
@@ -205,6 +196,7 @@ void
 get_args(int argc, char *argv[])
 {
 	int	 c, i, nvals, nchars;
+	size_t	len;
 	char	*grid, *p;
 	float	 numer, denom;
 
@@ -227,143 +219,179 @@ get_args(int argc, char *argv[])
 						!strcmp(p, drivers[i].alias))) {
 				strcpy(lang, drivers[i].name);
 				dev = drivers[i].dev;
-				dev->option('L', optarg);
+				dev->option('L', drivers[i].name);
 				break;
 			}
 		}
 	}
 
 
-	/* sum of all arguments */
-	while ((c = getopt(argc, argv, ARGSTRING)) != EOF) {
+	/* all option letters must be in this string */
+	/* not in this string: HIJQUu and non-alphabetic chars*/
+	while ((c = getopt(argc, argv, "AaB:b:C:cD:d:E:eFf:G:g:hi:jKkL:l:Mm:Nn:"
+					"OoPp:q:R:rS:s:Tt:VvWwX:x:Y:y:Z:z:?"))
+			!= EOF) {
 
-	  /* global (all drivers) option handling */
-	    switch (c) {
+		/* global (all drivers) option handling */
+		switch (c) {
 
-	    case 'h':	/* print version message for -h too */
-	    case 'V':
-		printf("fig2dev Version %s\n", PACKAGE_VERSION);
-		if (c == 'h')
-		    help_msg();
-		exit(0);
-		break;
-
-	    case 'D':	                /* depth filtering */
-		depth_option(optarg);
-		/* options with "continue" are not passed to the driver */
-		continue;
-
-	    case 'K':
-		/* adjust bounding box according to selected depth range
-		   given with '-D RANGE' option above */
-		adjust_boundingbox = true;
-		continue;
-
-	    case 'G':			/* Grid */
-		/* if major AND minor are specified */
-		if ((p = strchr(optarg,':'))) {
-		    /* parse minor grid spec first */
-		    if (p != optarg) {
-			if ((nvals=parse_gridspec(optarg, &numer, &denom,
-					&grid_minor_spacing, &nchars)) == 2)
-			    grid_minor_spacing = numer/denom;
-		    }
-		    /* now parse major grid spec */
-		    if ((nvals=parse_gridspec(p+1, &numer, &denom,
-				    &grid_major_spacing, &nchars)) == 2)
-			grid_major_spacing = numer/denom;
-		} else {
-		    /* only minor grid specified */
-		    if ((nvals=parse_gridspec(optarg, &numer, &denom,
-				    &grid_minor_spacing, &nchars)) == 2)
-			grid_minor_spacing = numer/denom;
-		    p = optarg-1;
-		}
-		/* now parse any units after grid spacing */
-		if ((size_t)nchars < strlen(p+1)) {
-		    grid = p+nchars+1;
-		    if ((strcmp(grid,"i") && strcmp(grid,"in") &&
-					strcmp(grid,"inch") &&
-				strcmp(grid,"f") && strcmp(grid,"ft") &&
-					strcmp(grid,"feet") &&
-				strcmp(grid,"c") && strcmp(grid,"cm") &&
-				strcmp(grid,"mm") && strcmp(grid,"m") )) {
-			grid_usage();
-			grid_major_spacing = grid_minor_spacing = 0.0;
+		case 'h':		/* print version message for -h too */
+		case 'V':
+			printf("fig2dev Version %s\n", PACKAGE_VERSION);
+			if (c == 'h')
+				help_msg();
+			exit(0);
 			break;
-		    }
-		    if (!strcmp(grid,"i") || !strcmp(grid,"in") ||
-				!strcmp(grid,"in"))
-			mult =  1.0;
-		    else if (!strcmp(grid,"f") || !strcmp(grid,"ft") ||
-				!strcmp(grid,"feet"))
-			mult = 12.0;
-		    else if (strcmp(grid,"c") == 0 || strcmp(grid,"cm") == 0)
-			mult = 1.0/2.54;
-		    else if (strcmp(grid,"mm") == 0)
-			mult = 1.0/25.4;
-		    else if (strcmp(grid,"m") == 0)
-			mult = 1.0/0.0254;
-		} else {
-		    mult = 1.0;	/* no units, 1:1 */
-		}
-		break;	/* error message given in genpstricks.c */
 
-	    case 'L':			/* set output language */
-		/* save language for gen{gif,jpg,pcx,xbm,xpm,ppm,tif} */
-		strncpy(lang, optarg, sizeof(lang)-1);
-		lang[sizeof lang - 1] = '\0';
-		/* override if a language already was set above */
-		dev = NULL;
-		for (i = 0; *drivers[i].name; ++i)
-		    if (!strcmp(lang, drivers[i].name))
-			dev = drivers[i].dev;
+		case 'D':                /* depth filtering */
+			depth_option(optarg);
+			/* options with "continue" are not passed to
+			   the driver */
+			continue;
+
+		case 'E':
+			if (!strcmp(optarg, "0") || !strcmp(optarg, "utf8") ||
+					!strcmp(optarg, "UTF8") ||
+					!strcmp(optarg, "utf-8"))
+				input_encoding = "UTF-8";
+			else if (!strcmp(optarg, "1"))
+				input_encoding = "ISO-8859-1";
+			else if (!strcmp(optarg, "2"))
+				input_encoding = "ISO-8859-2";
+			else
+				input_encoding = optarg;
+			continue;
+
+		case 'K':
+			/* adjust bounding box according to selected
+			   depth range given with '-D RANGE' option above */
+			adjust_boundingbox = true;
+			continue;
+
+		case 'G':		/* Grid */
+			/* if major AND minor are specified */
+			if ((p = strchr(optarg,':'))) {
+				/* parse minor grid spec first */
+				if (p != optarg) {
+					if ((nvals = parse_gridspec(optarg,
+							&numer, &denom,
+							&grid_minor_spacing,
+							&nchars)) == 2)
+						grid_minor_spacing =numer/denom;
+				}
+				/* now parse major grid spec */
+				if ((nvals = parse_gridspec(p+1, &numer, &denom,
+							&grid_major_spacing,
+							&nchars)) == 2)
+					grid_major_spacing = numer/denom;
+			} else {
+				/* only minor grid specified */
+				if ((nvals = parse_gridspec(optarg, &numer,
+							&denom,
+							&grid_minor_spacing,
+							&nchars)) == 2)
+					grid_minor_spacing = numer/denom;
+				p = optarg - 1;
+			}
+			/* now parse any units after grid spacing */
+			if ((size_t)nchars < strlen(p+1)) {
+				grid = p+nchars+1;
+				if ((strcmp(grid,"i") && strcmp(grid,"in") &&
+							strcmp(grid,"inch") &&
+							strcmp(grid,"f") &&
+							strcmp(grid,"ft") &&
+							strcmp(grid,"feet") &&
+							strcmp(grid,"c") &&
+							strcmp(grid,"cm") &&
+							strcmp(grid,"mm") &&
+							strcmp(grid,"m") )) {
+					grid_usage();
+					grid_major_spacing = 0.0;
+					grid_minor_spacing = 0.0;
+					break;
+				}
+				if (!strcmp(grid,"i") || !strcmp(grid,"in") ||
+						!strcmp(grid,"in"))
+					mult =  1.0;
+				else if (!strcmp(grid,"f") || !strcmp(grid,"ft")
+						|| !strcmp(grid,"feet"))
+					mult = 12.0;
+				else if (strcmp(grid,"c") == 0 ||
+						strcmp(grid,"cm") == 0)
+					mult = 1.0/2.54;
+				else if (strcmp(grid,"mm") == 0)
+					mult = 1.0/25.4;
+				else if (strcmp(grid,"m") == 0)
+					mult = 1.0/0.0254;
+			} else {
+				mult = 1.0;	/* no units, 1:1 */
+			}
+			break;	/* error message given in genpstricks.c */
+
+		case 'L':		/* set output language */
+			/* save language for gen{gif,jpg,pcx,xbm,xpm,ppm,tif} */
+			strncpy(lang, optarg, sizeof(lang)-1);
+			lang[sizeof lang - 1] = '\0';
+			/* override if a language already was set above */
+			dev = NULL;
+			for (i = 0; *drivers[i].name; ++i)
+				if (!strcmp(lang, drivers[i].name))
+					dev = drivers[i].dev;
+			if (!dev) {
+				fprintf(stderr,"Unknown graphics language %s\n",
+						lang);
+				fprintf(stderr, "Known languages are:\n");
+				/* display available languages - 23/01/90 */
+				for (i=0; *drivers[i].name; i++)
+					fprintf(stderr,"%s ",drivers[i].name);
+				fprintf(stderr,"\n");
+				exit(1);
+			}
+			break;	/* needed in genepic.c, genmp.c */
+
+		case 'm':		/* set magnification */
+			fontmag = mag = atof(optarg);
+			magspec = true;		/* user-specified */
+			continue;
+
+		case 's':		/* set default font size */
+			font_size = atof(optarg);
+			/* max size is checked in respective drivers */
+			if (font_size <= 0.0)
+				font_size = DEFAULT_FONT_SIZE;
+			continue;
+
+		case 'Z':		/* scale to requested size (inches/cm)*/
+			len = strlen(optarg);
+			errno = 0;
+			if (optarg[len - 1] == 'h' || optarg[0] == ':') {
+				/* height given */
+				max_dimension = strtod(optarg[0] == ':' ?
+						optarg + 1 : optarg, NULL);
+				maxdimspec = 3;
+			} else {
+				max_dimension = strtod(optarg, NULL);
+				maxdimspec = optarg[len - 1] == 'w' ||
+						optarg[len - 1] == ':' ? 2 : 1;
+			}
+			if (errno != 0	|| !isfinite(max_dimension)) {
+				fprintf(stderr, "Unable to parse dimension spec"
+						"ification (-Z %s)", optarg);
+				maxdimspec = 0;
+			}
+			continue;
+
+		case '?':		/* usage		*/
+			fprintf(stderr, Usage, prog);
+			exit(1);
+		}
+
+		/* pass options through to driver */
 		if (!dev) {
-		    fprintf(stderr,
-			    "Unknown graphics language %s\n", lang);
-		    fprintf(stderr,"Known languages are:\n");
-		    /* display available languages - 23/01/90 */
-		    for (i=0; *drivers[i].name; i++)
-			fprintf(stderr,"%s ",drivers[i].name);
-		    fprintf(stderr,"\n");
-		    exit(1);
+			fprintf(stderr, "No graphics language specified.\n");
+			exit(1);
 		}
-		break;	/* needed in genepic.c, genmp.c */
-
-	    case 'm':			/* set magnification */
-		fontmag = mag = atof(optarg);
-		magspec = true;		/* user-specified */
-		continue;
-
-	    case 's':			/* set default font size */
-		font_size = atof(optarg);
-		/* max size is checked in respective drivers */
-		if (font_size <= 0.0)
-		    font_size = DEFAULT_FONT_SIZE;
-		continue;
-
-	    case 'Z':			/* scale to requested size (inches/cm)*/
-		maxdimspec = true;
-		max_dimension = atof(optarg);
-		continue;
-
-#ifdef I18N
-	    case 'j':
-		support_i18n = true;
-		continue;
-#endif /* I18N */
-
-	    case '?':			/* usage		*/
-		fprintf(stderr,Usage,prog,prog);
-		exit(1);
-	    }
-
-	    /* pass options through to driver */
-	    if (!dev) {
-		fprintf(stderr, "No graphics language specified.\n");
-		exit(1);
-	    }
-	    dev->option(c, optarg);
+		dev->option(c, optarg);
 	}
 	/* adjust font size after option loop to make sure we have any -m first,
 	   which affects fontmag */
@@ -378,7 +406,7 @@ get_args(int argc, char *argv[])
 	if (magspec && maxdimspec) {
 		fputs("Must specify only one of -m (magnification) "
 				"and -Z (max dimension).\n",
-			stderr);
+				stderr);
 		exit(1);
 	}
 
@@ -394,16 +422,16 @@ parse_gridspec(char *string, float *numer,float *denom, float *spacing,
 {
 	int  numvals;
 
-	if ((numvals=sscanf(string,"%f/%f%n",numer,denom,nchrs)) == 2) {
-	    if (*numer < 0.0 || *denom < 0.0 || *denom == 0.0)
-		return 0;
-	    else
-		return numvals;
+	if ((numvals = sscanf(string, "%f/%f%n", numer, denom, nchrs)) == 2) {
+		if (*numer < 0.0 || *denom < 0.0 || *denom == 0.0)
+			return 0;
+		else
+			return numvals;
 	} else if ((numvals=sscanf(string,"%f%n",spacing,nchrs)) == 1) {
-	    if (*spacing < 0.0 || *spacing == 0.0)
-		return 0;
-	    else
-		return numvals;
+		if (*spacing < 0.0 || *spacing == 0.0)
+			return 0;
+		else
+			return numvals;
 	}
 
 	grid_usage();
@@ -413,12 +441,12 @@ parse_gridspec(char *string, float *numer,float *denom, float *spacing,
 static void
 grid_usage(void)
 {
-     fputs("Can't parse grid spec. Format is -G [minor_tick][:major_tick]unit ",
-	     stderr);
-     fputs("e.g. -G .5:2cm or -G 1/16:1/2in\n", stderr);
-     fputs("Allowable units are: ", stderr);
-     fputs("'i', 'in', 'ft', 'feet', 'c', 'cm', 'mm', 'm'.", stderr);
-     fputs("  Ignoring grid.\n", stderr);
+	fputs("Can't parse grid spec. Format is -G "
+			"[minor_tick][:major_tick]unit ", stderr);
+	fputs("e.g. -G .5:2cm or -G 1/16:1/2in\n", stderr);
+	fputs("Allowable units are: ", stderr);
+	fputs("'i', 'in', 'ft', 'feet', 'c', 'cm', 'mm', 'm'.", stderr);
+	fputs("  Ignoring grid.\n", stderr);
 }
 
 int
@@ -481,14 +509,22 @@ main(int argc, char *argv[])
 	/* if user has specified the maximum dimension,
 	   adjust mag to produce that */
 	if (maxdimspec) {
-		float maxdim;
+		int	figdim;
+
+		switch (maxdimspec) {
+		case 2:
+			figdim = urx -llx;
+			break;
+		case 3:
+			figdim = ury - lly;
+			break;
+		default:
+			figdim = MAX(urx - llx, ury - lly);
+		}
 
 		if (metric)
 			max_dimension /= 2.54;
-		/* find larger of width and height */
-		maxdim = MAX(urx - llx, ury - lly)/ppi;
-		/* override any mag specified in the Fig file */
-		mag = max_dimension/maxdim;
+		mag = max_dimension * ppi / figdim;
 	}
 	/*
 	 * If metric, adjust scale for difference between FIG PIX/CM (450) and
@@ -510,6 +546,7 @@ help_msg(void)
 {
 	int i, n;
 
+	fprintf(stdout, Usage, prog);
 	fputs(	"General Options (all drivers):\n"
 "  -L language	choose output language (this must be first, if the language\n"
 "                cannot be inferrred from the suffix of the output file)\n"
@@ -528,16 +565,16 @@ help_msg(void)
 "  -V          print fig2dev version number and exit\n"
 "  -D +/-list  include or exclude depths listed\n"
 "  -K          adjust bounding box according to selected depths\n"
-"                      given with '-D +/-list' option.\n"
+"                      given with '-D +/-list' option\n"
+"  -E enc      set the character encoding of the input file\n"
 "  -G minor[:major][unit]    draw light gray grid with thin/thick lines at\n"
 "                minor/major units (e.g., -G .25:1cm)\n"
-#ifdef I18N
-"  -j          enable i18n facility\n"
-#endif
 "  -m mag      set magnification.  This may not be used with the -Z option\n"
 "  -s size     set default font size in points\n"
-"  -Z maxdim   Scale the figure so that the maximum dimension (width or height)\n"
-"                is maxdim inches/cm.  This may not be used with the -m option"
+"  -Z maxdim   scale the figure so that the maximum dimension (width or height)\n"
+"                is maxdim inches/cm.  This may not be used with the -m option \n"
+"  -Z <width>w   append 'w' or a colon (':') to specify the width, append 'h' or\n"
+"  -Z <height>h  prepend a colon to specify the height\n"
 	);
 
 	/* Vector formats */
@@ -571,22 +608,10 @@ help_msg(void)
 		if (dev == NULL || !strcmp(lang, "dxf")) {
 			puts(
 "DXF Options:\n"
-#ifdef A4
-"  -a          select ANSI A paper size instead of the default ISO A4\n"
-#else
-"  -a          select ISO A4 paper size instead of the default ANSI A\n"
-#endif
-#ifdef IBMGEC
-"  -c          generate instructions for IBM 6180 plotter without IBM-GEC\n"
-#else
-"  -c          generate instructions for IBM 6180 plotter with IBM-GEC\n"
-#endif
-"  -d xll,yll,xur,yur    restrict plotting to area specified by coords\n"
 "  -f fontfile load text character specs from table in file\n"
 "  -l pattfile load patterns for pattern fill from file\n"
 "  -P          rotate figure to portrait (default is landscape)\n"
-"  -p penfile  load plotter pen specs from file\n"
-"  -v          print figure upside-down in portrait or backwards in landscape"
+"  -p penfile  load plotter pen specs from file"
 			);
 		}
 
@@ -604,13 +629,13 @@ help_msg(void)
 			puts(
 "EPS, PDFTEX, PSTEX and PDF (if -P is not given) Options:\n"
 "  -A          add ASCII preview (produce EPSI file; not for PDF output)\n"
-"  -a          do not output user's login name (anonymous)\n"
 "  -B \"Wx [Wy X0 Y0]\"    force width, height and origin in absolute coordinates\n"
 "  -b width    specify width of blank border around figure (points, 1/72 inch)\n"
 "  -C dummyarg add color TIFF preview (for Microsoft apps; not for PDF output)\n"
 "  -F          use correct font sizes (points instead of 1/80inch)\n"
 "  -f font     set default font\n"
 "  -g color    set background color\n"
+"  -j          enable composite fonts for japanese or korean text\n"
 "  -N          convert all colors to grayscale\n"
 "  -n name     set title part of PostScript output to name\n"
 "  -R \"Wx [Wy X0 Y0]\"    force width, height and origin in relative\n"
@@ -625,8 +650,6 @@ help_msg(void)
 			puts(
 "EPIC, EEPIC and EEPICEMU Options:\n"
 "  -d scale    divide size of arrowheads by scale\n"
-"  -E num      set encoding for text translation (0 = no translation,\n"
-"                1 = ISO-8859-1, 2 = ISO-8859-2; default 1)\n"
 #ifdef NFSS
 "  -F          do not set font family, series or shape\n"
 #endif /* NFSS */
@@ -689,8 +712,6 @@ help_msg(void)
 "LaTeX Options:\n"
 "  -b width    specify width of blank border around figure (1/72 inch)\n"
 "  -d dmag     set separate magnification for length of line dashes to dmag\n"
-"  -E num      set encoding for text translation (0 = no translation,\n"
-"                1 = ISO-8859-1, 2 = ISO-8859-2; default 1)\n"
 #ifdef NFSS
 "  -F          do not set font family/series/shape\n"
 #endif
@@ -742,9 +763,6 @@ help_msg(void)
 		if (dev == NULL || !strcmp(lang, "pictex")) {
 			puts(
 "PICTeX Options:\n"
-"  -a          don't output user's login name (anonymous)\n"
-"  -E num      set encoding for text translation (0 = no translation,\n"
-"                1 = ISO-8859-1, 2 = ISO-8859-2; default 1)\n"
 "  -f font     set default font\n"
 "  -l dimen    set line thickness to TeX dimen\n"
 "  -p symbol   set plot symbol (default \\tencirc\\symbol{'160})\n"
@@ -759,14 +777,12 @@ help_msg(void)
 "  -C num      do not issue color commands for color num (0 = black,\n"
 "                1 = blue, etc., see xfig color chooser)\n"
 "  -e          do not try to be compatible with epic/eepic\n"
-"  -E num      set encoding for text translation (0 = no translation,\n"
-"                1 = ISO-8859-1, 2 = ISO-8859-2; default 1)\n"
 "  -F          do not set font family/series/shape\n"
 "  -f font     set default font\n"
 "  -i dir      prepend the string \"dir\" to included graphics files\n"
 "  -o          do not set the font size\n"
 "  -O          do not quote characters special to TeX/LaTeX\n"
-"                (effectively, set the text-flag \"special-text\" for all text)\n"
+"                (effectively, set the text flag \"TeX\" for all text)\n"
 "  -P          pagemode, generate stand-alone LaTeX-file\n"
 "  -R num      use LaTeX-arrows for arrow-type num\n"
 "  -r          use LaTeX-arrows for all arrows\n"
@@ -780,7 +796,6 @@ help_msg(void)
 			puts(
 "PostScript and PDF (if -P is given) Options:\n"
 "  -A          add ASCII (EPSI) preview, not for PDF output\n"
-"  -a          do not output user's login name (anonymous)\n"
 "  -b width    specify width of blank border around figure (1/72 inch)\n"
 "  -C dummyarg add color TIFF preview (for Microsoft apps), not for PDF output\n"
 "  -c          center figure on page\n"
@@ -788,6 +803,7 @@ help_msg(void)
 "  -F          use correct font sizes (points instead of 1/80inch)\n"
 "  -f font     set default font\n"
 "  -g color    set background color\n"
+"  -j          enable composite fonts for japanese or korean text\n"
 "  -l dummyarg landscape mode (dummy argument required after \"-l\")\n"
 "  -M          generate multiple pages for large figure\n"
 "  -N          convert all colors to grayscale\n"
@@ -832,8 +848,6 @@ help_msg(void)
 			puts(
 "PSTEX_T and PDFTEX_T Options:\n"
 "  -b width    specify width of blank border around figure (1/72 inch)\n"
-"  -E num      set encoding for text translation (0 = no translation,\n"
-"                1 = ISO-8859-1, 2 = ISO-8859-2; default 1)\n"
 #ifdef NFSS
 "  -F          do not set font family, series or shape, so you can\n"
 "                set it from latex\n"
@@ -870,12 +884,10 @@ help_msg(void)
 "  -b width    specify width of blank border around figure (1/72 inch)\n"
 "  -C num      do not issue color commands for color num (0 = black,\n"
 "                1 = blue, etc., see xfig color chooser)\n"
-"  -E num      set encoding for text translation (0 = no translation,\n"
-"                1 = ISO-8859-1, 2 = ISO-8859-2; default 1)\n"
 "  -F          do not set font family/series/shape\n"
 "  -i dir      prepend the string \"dir\" to included graphics files\n"
 "  -O          do not quote characters special to TeX/LaTeX\n"
-"                (effectively, set the text-flag \"special-text\" for all text)\n"
+"                (effectively, set the text flag \"TeX\" for all text)\n"
 "  -o          do not set the font size\n"
 "  -P          pagemode, generate stand-alone LaTeX-file\n"
 "  -T          only use TeX fonts (even when PS-fonts are specified)\n"
@@ -944,46 +956,46 @@ static int compound_dump(F_compound *com, struct obj_rec *array,
 	F_text		*t;
 
 	for (c = com->compounds; c != NULL; c = c->next)
-	  count = compound_dump(c, array, count, dev);
+		count = compound_dump(c, array, count, dev);
 	for (a = com->arcs; a != NULL; a = a->next) {
-	  if (array) {
-		array[count].gendev = dev->arc;
-		array[count].obj = (char *)a;
-		array[count].depth = a->depth;
-	  }
-	  count += 1;
+		if (array) {
+			array[count].gendev = (void(*)(void *))dev->arc;
+			array[count].obj = (void *)a;
+			array[count].depth = a->depth;
+		}
+		count += 1;
 	}
 	for (e = com->ellipses; e != NULL; e = e->next) {
-	  if (array) {
-		array[count].gendev = dev->ellipse;
-		array[count].obj = (char *)e;
-		array[count].depth = e->depth;
-	  }
-	  count += 1;
+		if (array) {
+			array[count].gendev = (void(*)(void *))dev->ellipse;
+			array[count].obj = (void *)e;
+			array[count].depth = e->depth;
+		}
+		count += 1;
 	}
 	for (l = com->lines; l != NULL; l = l->next) {
-	  if (array) {
-		array[count].gendev = dev->line;
-		array[count].obj = (char *)l;
-		array[count].depth = l->depth;
-	  }
-	  count += 1;
+		if (array) {
+			array[count].gendev = (void(*)(void *))dev->line;
+			array[count].obj = (void *)l;
+			array[count].depth = l->depth;
+		}
+		count += 1;
 	}
 	for (s = com->splines; s != NULL; s = s->next) {
-	  if (array) {
-		array[count].gendev = dev->spline;
-		array[count].obj = (char *)s;
-		array[count].depth = s->depth;
-	  }
-	  count += 1;
+		if (array) {
+			array[count].gendev = (void(*)(void *))dev->spline;
+			array[count].obj = (void *)s;
+			array[count].depth = s->depth;
+		}
+		count += 1;
 	}
 	for (t = com->texts; t != NULL; t = t->next) {
-	  if (array) {
-		array[count].gendev = dev->text;
-		array[count].obj = (char *)t;
-		array[count].depth = t->depth;
-	  }
-	  count += 1;
+		if (array) {
+			array[count].gendev = (void(*)(void *))dev->text;
+			array[count].obj = (void *)t;
+			array[count].depth = t->depth;
+		}
+		count += 1;
 	}
 	return count;
 }
@@ -1004,8 +1016,8 @@ gendev_objects(F_compound *objects, struct driver *dev)
 	/* dump object pointers to an array */
 	obj_count = compound_dump(objects, 0, 0, dev);
 	if (!obj_count) {
-	    fprintf(stderr, "fig2dev: No objects in Fig file\n");
-	    return -1;
+		fprintf(stderr, "fig2dev: No objects in Fig file\n");
+		return -1;
 	}
 	rec_array = (struct obj_rec *)malloc(obj_count*sizeof(struct obj_rec));
 	(void)compound_dump(objects, rec_array, 0, dev);
@@ -1022,8 +1034,8 @@ gendev_objects(F_compound *objects, struct driver *dev)
 
 	/* generate objects in sorted order */
 	for (r = rec_array; r<rec_array+obj_count; r++)
-	  if (depth_filter(r->depth))
-	    (*(r->gendev))(r->obj);
+		if (depth_filter(r->depth))
+			(*(r->gendev))(r->obj);
 
 	/* generate trailer */
 	status = (*dev->end)();
@@ -1057,12 +1069,15 @@ gendev_nogrid(float major, float minor)
 static void
 depth_usage(void)
 {
-    fprintf(stderr,"%s: help for -D option:\n",prog);
-    fprintf(stderr,"  -D +rangelist  means keep only depths in rangelist.\n");
-    fprintf(stderr,"  -D -rangelist  means keep all depths but those in rangelist.\n");
-    fprintf(stderr,"  Rangelist can be a list of numbers or ranges of numbers, e.g.:\n");
-    fprintf(stderr,"    10,40,55,60:70,99\n");
-    exit(1);
+	fprintf(stderr, "%s: help for -D option:\n", prog);
+	fputs("  -D +rangelist  means keep only depths in rangelist.\n",
+			stderr);
+	fputs("  -D -rangelist  means keep all depths but those in "
+			"rangelist.\n", stderr);
+	fputs("  Rangelist can be a list of numbers or ranges of numbers, "
+			"e.g.:\n", stderr);
+	fputs("    10,40,55,60:70,99\n", stderr);
+	exit(1);
 }
 
 void
@@ -1079,25 +1094,25 @@ depth_option(char *s)
 	}
 
 	for (d = depth_opt; depth_index < NUMDEPTHS && *s; ++depth_index, d++) {
-	    if (*s == ',') ++s;
-	    d->d1 = d->d2 = -1;
-	    d->d1 = strtol(s,&s,10);
-	    if (d->d1 < 0)
-		depth_usage();
-	    switch(*s) {		/* what's the delim? */
-	    case ':':			/* parse a range */
-		d->d2 = strtol(s+1,&s,10);
-		if (d->d2 < d->d1)
-		    depth_usage();
-		break;
-	    case ',':			/* just start the next one */
-		s++;
-		break;
-	    }
+		if (*s == ',') ++s;
+		d->d1 = d->d2 = -1;
+		d->d1 = strtol(s,&s,10);
+		if (d->d1 < 0)
+			depth_usage();
+		switch(*s) {		/* what's the delim? */
+		case ':':			/* parse a range */
+			d->d2 = strtol(s+1,&s,10);
+			if (d->d2 < d->d1)
+				depth_usage();
+			break;
+		case ',':			/* just start the next one */
+			s++;
+			break;
+		}
 	}
 	if (depth_index >= NUMDEPTHS) {
-	    fprintf(stderr,"%s: Too many -D values!\n",prog);
-	    exit(1);
+		fprintf(stderr,"%s: Too many -D values!\n",prog);
+		exit(1);
 	}
 	d->d1 = -1;
 }
@@ -1108,14 +1123,14 @@ depth_filter(int obj_depth)
 	struct depth_opts *d;
 
 	if (depth_index <= 0)		/* no filters were set up */
-	    return 1;
+		return 1;
 	for (d = depth_opt; d->d1 >= 0; d++)
-	    if (d->d2 >= 0) {		/* it's a range comparison */
-		if (obj_depth >= d->d1 && obj_depth <= d->d2)
-		    return (depth_op=='+')? 1:0;
-	    } else {			/* it's a single-value comparison */
-		if (d->d1 == obj_depth)
-		    return (depth_op=='+')? 1:0;
-	    }
+		if (d->d2 >= 0) {	/* it's a range comparison */
+			if (obj_depth >= d->d1 && obj_depth <= d->d2)
+				return (depth_op=='+')? 1:0;
+		} else {		/* it's a single-value comparison */
+			if (d->d1 == obj_depth)
+				return (depth_op=='+')? 1:0;
+		}
 	return (depth_op=='-')? 1:0;
 }
