@@ -2,7 +2,8 @@
  * Fig2dev: Translate Fig code to various Devices
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2019 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,7 +16,12 @@
  *
  */
 
+#ifndef LOCALMATH_H
+#define LOCALMATH_H
+
 extern void arc_tangent(double x1, double y1, double x2, double y2,
 				int direction, double *x, double *y);
 extern void arc_tangent_int(double x1, double y1, double x2, double y2,
 				int direction, int *x, int *y);
+
+#endif /* LOCALMATH_H */
