@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2019 by Thomas Loimer
+ * Parts Copyright (c) 2015-2023 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -18,6 +18,8 @@
 
 #ifndef READ_H
 #define READ_H
+#include <stdio.h>
+#include "object.h"
 
 extern int	read_fig(char *file_name, F_compound *obj);
 extern int	readfp_fig(FILE *fp, F_compound *obj);
