@@ -92,8 +92,7 @@ free_point_array(F_pos *pts)
 
 
 static bool
-init_point_array(init_size, step_size)
-    int		    init_size, step_size;
+init_point_array(int init_size, int step_size)
 {
     npoints = 0;
     max_points = init_size;
@@ -580,6 +579,11 @@ create_line_with_spline(F_spline *s)
 	  ptr = ptr->next;
 	}
     }
+  line->num_points = npoints;
+  line->last[0].x = points[i - 1].x;
+  line->last[0].y = points[i - 1].y;
+  line->last[1].x = points[i - 2].x;
+  line->last[1].y = points[i - 2].y;
 
   free_point_array(points);
   npoints = 0;
